﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using UnityEngine;
using UnityEngine.Events;

namespace CarMod
{
    public class CarMod
    {
        public static string tag = " -CM";
        public static void Main()
        {
            Debug.Log(DateTime.Now.ToString("dd.MM.yyyy\nhh:mm:ss"));
            Liquid.Register("GASOLINE", new Gasoline());
            LiquidMixingController.MixInstructions.Add(new LiquidMixInstructions(Liquid.GetLiquid(Oil.ID), Liquid.GetLiquid(Chemistry.Tritium.ID), Liquid.GetLiquid(Chemistry.BeverageM04.ID), Liquid.GetLiquid("GASOLINE"))
            {
                RatePerSecond = 0.2f
            });
            ModAPI.Register(
                new Modification
                {
                    OriginalItem = ModAPI.FindSpawnable("Car"),
                    NameOverride = "Audi A4" + tag,
                    NameToOrderByOverride = "!",
                    DescriptionOverride = "Car produced since 1994 by the German car manufacturer Audi, a subsidiary of the Volkswagen Group.",
                    CategoryOverride = ModAPI.FindCategory("Vehicles"),
                    ThumbnailOverride = ModAPI.LoadSprite("Textures/Cars/Audi A4/Thumb.png"),
                    AfterSpawn = (Instance) =>
                    {
                        Transform body = Instance.transform.GetChild(0);
                        bool firstSpawn = body.localPosition == Vector3.zero;
                        if (firstSpawn)
                        {
                            body.GetComponent<CarBehaviour>().MotorSpeed = -4600f;
                        }
                        BoxCollider2D[] boxColliders = body.GetComponents<BoxCollider2D>();
                        boxColliders[0].offset = new Vector2(0.0003607571f, -0.6723801f);
                        boxColliders[0].size = new Vector2(0.7148612f, 0.342592f);
                        boxColliders[1].offset = new Vector2(-0.3000616f, 0.09522128f);
                        boxColliders[1].size = new Vector2(0.1689302f, 1.26843f);
                        boxColliders[2].offset = new Vector2(-1.191312f, -0.6707045f);
                        boxColliders[2].size = new Vector2(0.6972498f, 0.3421229f);
                        boxColliders[3].offset = new Vector2(-1.631744f, 0.01107115f);
                        boxColliders[3].size = new Vector2(0.2636082f, 1.152815f);
                        body.GetComponent<PolygonCollider2D>().SetPath(0, new Vector2[]
                        {
                            new Vector2(3.060745f, -0.9329169f),
                            new Vector2(3.528571f, -0.8354951f),
                            new Vector2(3.337887f, -0.0630527f),
                            new Vector2(1.614175f, 0.1910262f),
                            new Vector2(0.8274078f, 0.7162807f),
                            new Vector2(0.1718019f, 0.9288237f),
                            new Vector2(-1.311635f, 0.9285715f),
                            new Vector2(-2.571465f, 0.3875467f),
                            new Vector2(-2.570911f, -0.1323804f),
                            new Vector2(-3.413413f, -0.1402977f),
                            new Vector2(-3.415922f, -0.7100658f),
                            new Vector2(-2.750413f, -0.9304728f)
                        });
                        body.GetComponent<PolygonCollider2D>().SetPath(1, new Vector2[]
                        {
                            new Vector2(-1.816524f, 0.1034421f),
                            new Vector2(-1.819524f, 0.2552218f),
                            new Vector2(-1.559001f, 0.7247119f),
                            new Vector2(-1.153004f, 0.8431288f),
                            new Vector2(0.111502f, 0.8465484f),
                            new Vector2(0.5939376f, 0.7035104f),
                            new Vector2(1.441056f, 0.1544568f),
                            new Vector2(1.441444f, -0.813762f),
                            new Vector2(-1.157408f, -0.8089445f)
                        });
                        body.GetChild(0).localPosition = new Vector3(0.1000073f, -0.0142924f);

                        WheelJoint2D[] wheels = body.GetComponents<WheelJoint2D>();
                        wheels[0].anchor = new Vector2(2.36f, -0.806f);
                        wheels[1].anchor = new Vector2(-2.068f, -0.806f);

                        if (firstSpawn)
                        {
                            wheels[0].connectedBody.transform.localPosition = new Vector3(2.36f, -0.806f);
                            wheels[1].connectedBody.transform.localPosition = new Vector3(-2.068f, -0.806f);
                        }

                        GameObject headlight = new GameObject("Headlight");
                        headlight.transform.parent = body;
                        headlight.transform.rotation = body.rotation;
                        headlight.transform.localPosition = new Vector3(3.214f, -0.2f);
                        headlight.transform.localScale = new Vector3(1f, 1f);
                        SpriteRenderer headlightRenderer = headlight.GetOrAddComponent<SpriteRenderer>();
                        headlightRenderer.sprite = ModAPI.LoadSprite("Textures/Cars/Audi A4/Headlight.png");
                        headlightRenderer.sortingOrder = body.GetComponent<SpriteRenderer>().sortingOrder + 1;
                        headlight.AddComponent<Optout>();
                        ModAPI.CreateLight(headlight.transform, Color.white, 3.5f, 1.5f);

                        GameObject taillight = new GameObject("Taillight");
                        taillight.transform.parent = body;
                        taillight.transform.rotation = body.rotation;
                        taillight.transform.localPosition = new Vector3(-3.257f, 0.057f);
                        taillight.transform.localScale = new Vector3(1f, 1f);
                        SpriteRenderer taillightRenderer = taillight.GetOrAddComponent<SpriteRenderer>();
                        taillightRenderer.sprite = ModAPI.LoadSprite("Textures/Cars/Audi A4/Taillight.png");
                        taillightRenderer.sortingOrder = body.GetComponent<SpriteRenderer>().sortingOrder + 1;
                        taillight.AddComponent<Optout>();
                        ModAPI.CreateLight(taillight.transform, Color.red, 3.5f, 1.5f);

                        Transform bonnet = Instance.transform.GetChild(3);
                        bonnet.GetComponent<HingeJoint2D>().anchor = new Vector2(-0.8157148f, 0.1143804f);
                        if (firstSpawn)
                        {
                            bonnet.localPosition = new Vector3(2.471424f, 0.0857097f);
                        }
                        bonnet.GetComponent<PolygonCollider2D>().SetPath(0, new Vector2[]
                        {
                            new Vector2(0.5923996f, -0.01134229f),
                            new Vector2(-0.4646941f, 0.1571429f),
                            new Vector2(-0.8285714f, 0.1571429f),
                            new Vector2(-0.8285714f, 0.1023642f),
                            new Vector2(0.5709956f, -0.1571429f),
                            new Vector2(0.8285714f, -0.1571429f)
                        });

                        Transform boot = Instance.transform.GetChild(4);
                        boot.GetComponent<HingeJoint2D>().anchor = new Vector2(0.41f, 0.2f);
                        if (firstSpawn)
                        {
                            boot.localPosition = new Vector3(-3.04f, 0.13f);
                        }
                        BoxCollider2D collider1 = boot.GetComponent<BoxCollider2D>();
                        collider1.offset = new Vector2(-0.3952925f, -0.1574069f);
                        collider1.size = new Vector2(0.1237006f, 0.08763137f);
                        boot.GetComponent<PolygonCollider2D>().SetPath(0, new Vector2[]
                        {
                            new Vector2(-0.3127339f, 0.1679793f),
                            new Vector2(-0.3698192f, -0.258925f),
                            new Vector2(-0.3199155f, -0.2592466f),
                            new Vector2(-0.2476237f, 0.1079744f),
                            new Vector2(0.4592464f, 0.1750954f),
                            new Vector2(0.4560386f, 0.2571429f)
                        });

                        Transform frontDoor = Instance.transform.GetChild(5);
                        if (firstSpawn)
                        {
                            frontDoor.localPosition = new Vector3(0.500002f, 0.0142789f);
                        }
                        frontDoor.GetComponent<PolygonCollider2D>().SetPath(0, new Vector2[]
                        {
                            new Vector2(0.4526816f, 0.543638f),
                            new Vector2(-0.3762484f, 0.857028f),
                            new Vector2(-0.9714286f, 0.8571429f),
                            new Vector2(-0.7434942f, 0.0252445f),
                            new Vector2(-0.7440699f, -0.8571429f),
                            new Vector2(0.9714286f, -0.8571429f),
                            new Vector2(0.9737331f, 0.04973945f)
                        });

                        Transform backDoor = Instance.transform.GetChild(6);
                        if (firstSpawn)
                        {
                            backDoor.localPosition = new Vector3(-1.042857f, 0.0142789f);
                        }
                        backDoor.GetComponent<PolygonCollider2D>().SetPath(0, new Vector2[]
                        {
                            new Vector2(-0.4491169f, 0.7937872f),
                            new Vector2(-0.7993343f, 0.2554604f),
                            new Vector2(-0.8f, 0.04698031f),
                            new Vector2(-0.1085917f, -0.8563237f),
                            new Vector2(0.8f, -0.8571429f),
                            new Vector2(0.8f, -0.1784481f),
                            new Vector2(0.5348876f, 0.8641579f)
                        });

                        if (firstSpawn)
                        {
                            Instance.transform.GetChild(7).localPosition = new Vector3(2.2f, -0.25f);
                            Instance.transform.GetChild(8).localPosition = new Vector3(-2.3f, -0.3f);
                        }

                        if (firstSpawn)
                        {
                            body.GetComponent<RandomCarTextureBehaviour>().chosenIndex = UnityEngine.Random.Range(0, 5);
                        }
                        body.GetComponent<RandomCarTextureBehaviour>().Textures = new RandomCarTextureBehaviour.CarSprites[]
                        {
                            new RandomCarTextureBehaviour.CarSprites
                            {
                                Body = ModAPI.LoadSprite("Textures/Cars/Audi A4/BlackColor/Body.png"),
                                Background = ModAPI.LoadSprite("Textures/Cars/Audi A4/BlackColor/Background.png"),
                                Bonnet = ModAPI.LoadSprite("Textures/Cars/Audi A4/BlackColor/Bonnet.png"),
                                Boot = ModAPI.LoadSprite("Textures/Cars/Audi A4/BlackColor/Boot.png"),
                                FrontDoor = ModAPI.LoadSprite("Textures/Cars/Audi A4/BlackColor/FrontDoor.png"),
                                BackDoor = ModAPI.LoadSprite("Textures/Cars/Audi A4/BlackColor/BackDoor.png")
                            },
                            new RandomCarTextureBehaviour.CarSprites
                            {
                                Body = ModAPI.LoadSprite("Textures/Cars/Audi A4/BlueColor/Body.png"),
                                Background = ModAPI.LoadSprite("Textures/Cars/Audi A4/BlueColor/Background.png"),
                                Bonnet = ModAPI.LoadSprite("Textures/Cars/Audi A4/BlueColor/Bonnet.png"),
                                Boot = ModAPI.LoadSprite("Textures/Cars/Audi A4/BlueColor/Boot.png"),
                                FrontDoor = ModAPI.LoadSprite("Textures/Cars/Audi A4/BlueColor/FrontDoor.png"),
                                BackDoor = ModAPI.LoadSprite("Textures/Cars/Audi A4/BlueColor/BackDoor.png")
                            },
                            new RandomCarTextureBehaviour.CarSprites
                            {
                                Body = ModAPI.LoadSprite("Textures/Cars/Audi A4/GrayColor/Body.png"),
                                Background = ModAPI.LoadSprite("Textures/Cars/Audi A4/GrayColor/Background.png"),
                                Bonnet = ModAPI.LoadSprite("Textures/Cars/Audi A4/GrayColor/Bonnet.png"),
                                Boot = ModAPI.LoadSprite("Textures/Cars/Audi A4/GrayColor/Boot.png"),
                                FrontDoor = ModAPI.LoadSprite("Textures/Cars/Audi A4/GrayColor/FrontDoor.png"),
                                BackDoor = ModAPI.LoadSprite("Textures/Cars/Audi A4/GrayColor/BackDoor.png")
                            },
                            new RandomCarTextureBehaviour.CarSprites
                            {
                                Body = ModAPI.LoadSprite("Textures/Cars/Audi A4/GreenColor/Body.png"),
                                Background = ModAPI.LoadSprite("Textures/Cars/Audi A4/GreenColor/Background.png"),
                                Bonnet = ModAPI.LoadSprite("Textures/Cars/Audi A4/GreenColor/Bonnet.png"),
                                Boot = ModAPI.LoadSprite("Textures/Cars/Audi A4/GreenColor/Boot.png"),
                                FrontDoor = ModAPI.LoadSprite("Textures/Cars/Audi A4/GreenColor/FrontDoor.png"),
                                BackDoor = ModAPI.LoadSprite("Textures/Cars/Audi A4/GreenColor/BackDoor.png")
                            },
                            new RandomCarTextureBehaviour.CarSprites
                            {
                                Body = ModAPI.LoadSprite("Textures/Cars/Audi A4/RedColor/Body.png"),
                                Background = ModAPI.LoadSprite("Textures/Cars/Audi A4/RedColor/Background.png"),
                                Bonnet = ModAPI.LoadSprite("Textures/Cars/Audi A4/RedColor/Bonnet.png"),
                                Boot = ModAPI.LoadSprite("Textures/Cars/Audi A4/RedColor/Boot.png"),
                                FrontDoor = ModAPI.LoadSprite("Textures/Cars/Audi A4/RedColor/FrontDoor.png"),
                                BackDoor = ModAPI.LoadSprite("Textures/Cars/Audi A4/RedColor/BackDoor.png")
                            }
                        };

                        CarOpticsBehaviour carOptics = body.gameObject.GetOrAddComponent<CarOpticsBehaviour>();
                        carOptics.Headlights = new GameObject[]
                        {
                            headlight,
                            taillight
                        };

                        NoCollide noCollide = Instance.GetComponent<NoCollide>();
                        noCollide.NoCollideSetA = Instance.GetComponentsInChildren<Collider2D>();
                        noCollide.NoCollideSetB = Instance.GetComponentsInChildren<Collider2D>();
                    }
                }
            );
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Car"),
                    NameOverride = "Lada 2104" + tag,
                    NameToOrderByOverride = "!",
                    DescriptionOverride = "Soviet and Russian rear-wheel drive car of the III group of a small class with a station wagon body. Developed at the Volga Automobile Plant. Produced from 1982 to 2006.",
                    CategoryOverride = ModAPI.FindCategory("Vehicles"),
                    ThumbnailOverride = ModAPI.LoadSprite("Textures/Cars/Lada 2104/Thumb.png"),
                    AfterSpawn = (Instance) =>
                    {
                        Transform body = Instance.transform.GetChild(0);
                        bool firstSpawn = body.localPosition == Vector3.zero;
                        if (firstSpawn)
                        {
                            body.GetComponent<CarBehaviour>().MotorSpeed = -4200f;
                        }
                        BoxCollider2D[] boxColliders = body.GetComponents<BoxCollider2D>();
                        boxColliders[0].offset = new Vector2(-0.9935724f, -0.6366533f);
                        boxColliders[0].size = new Vector2(0.7319374f, 0.3867753f);
                        boxColliders[1].offset = new Vector2(-1.412266f, 0.07183601f);
                        boxColliders[1].size = new Vector2(0.2066746f, 1.086045f);
                        boxColliders[2].offset = new Vector2(0.2173646f, -0.6447749f);
                        boxColliders[2].size = new Vector2(0.7642735f, 0.4010276f);
                        boxColliders[3].offset = new Vector2(-0.09891982f, 0.1281012f);
                        boxColliders[3].size = new Vector2(0.200361f, 1.260856f);
                        body.GetComponent<PolygonCollider2D>().SetPath(0, new Vector2[]
                        {
                            new Vector2(3.339788f, -0.04188037f),
                            new Vector2(3.269659f, -0.04320218f),
                            new Vector2(3.203921f, -0.2113113f),
                            new Vector2(1.57134f, -0.09922314f),
                            new Vector2(1.571429f, 0.1289061f),
                            new Vector2(1.486577f, 0.128993f),
                            new Vector2(1.344451f, 0.4632163f),
                            new Vector2(0.9703096f, 0.9293668f),
                            new Vector2(-2.261764f, 0.9285715f),
                            new Vector2(-3.085149f, 0.04257962f),
                            new Vector2(-3.285778f, -0.5570759f),
                            new Vector2(-2.972294f, -0.8398728f),
                            new Vector2(-1.26619f, -0.9285724f),
                            new Vector2(3.022087f, -0.9289024f)
                        });
                        body.GetComponent<PolygonCollider2D>().SetPath(1, new Vector2[]
                        {
                            new Vector2(-0.9709282f, -0.8142473f),
                            new Vector2(-1.545203f, -0.1505785f),
                            new Vector2(-1.342683f, 0.8427334f),
                            new Vector2(0.8568147f, 0.8430069f),
                            new Vector2(1.222358f, 0.4639388f),
                            new Vector2(1.461143f, -0.08030061f),
                            new Vector2(1.456752f, -0.8142435f)
                        });
                        body.GetChild(0).localPosition = new Vector3(0.1285647f, -0.0571414f);

                        WheelJoint2D[] wheels = body.GetComponents<WheelJoint2D>();
                        wheels[0].anchor = new Vector2(2.35f, -0.9f);
                        wheels[0].connectedBody.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("Textures/Cars/Lada 2104/Wheel.png");
                        wheels[0].connectedBody.GetComponent<CircleCollider2D>().radius = 0.4857143f;
                        wheels[1].anchor = new Vector2(-1.7f, -0.9f);
                        wheels[1].connectedBody.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("Textures/Cars/Lada 2104/Wheel.png");
                        wheels[1].connectedBody.GetComponent<CircleCollider2D>().radius = 0.4857143f;

                        if (firstSpawn)
                        {
                            wheels[0].connectedBody.transform.localPosition = new Vector3(2.35f, -0.9f);
                            wheels[1].connectedBody.transform.localPosition = new Vector3(-1.7f, -0.9f);
                        }

                        LightSprite headlight = ModAPI.CreateLight(body, Color.white, 3.5f, 1.5f);
                        headlight.transform.localPosition = new Vector3(3.23f, -0.403f, 0f);

                        GameObject taillight = new GameObject("Taillight");
                        taillight.transform.parent = body;
                        taillight.transform.rotation = body.rotation;
                        taillight.transform.localPosition = new Vector3(-3.0857f, -0.0286f, 0f);
                        taillight.transform.localScale = new Vector3(1f, 1f, 1f);
                        SpriteRenderer taillightRenderer = taillight.GetOrAddComponent<SpriteRenderer>();
                        taillightRenderer.sprite = ModAPI.LoadSprite("Textures/Cars/Lada 2104/Taillight.png");
                        taillightRenderer.sortingOrder = body.GetComponent<SpriteRenderer>().sortingOrder + 1;
                        taillight.AddComponent<Optout>();
                        ModAPI.CreateLight(taillight.transform, Color.red, 3.5f, 1.5f);

                        GameObject turnSignal1 = new GameObject("Turn Signal 1");
                        turnSignal1.transform.parent = body;
                        turnSignal1.transform.rotation = body.rotation;
                        turnSignal1.transform.localPosition = new Vector3(-3.1287f, -0.1572f, 0f);
                        turnSignal1.transform.localScale = new Vector3(1f, 1f, 1f);
                        SpriteRenderer turnSignal1Renderer = turnSignal1.GetOrAddComponent<SpriteRenderer>();
                        turnSignal1Renderer.sprite = ModAPI.LoadSprite("Textures/Cars/Lada 2104/TurnSignal1.png");
                        turnSignal1Renderer.sortingOrder = body.GetComponent<SpriteRenderer>().sortingOrder + 1;
                        turnSignal1.AddComponent<Optout>();
                        ModAPI.CreateLight(turnSignal1.transform, new Color32(255, 146, 0, 255), 1f, 1f);

                        GameObject turnSignal2 = new GameObject("Turn Signal 2");
                        turnSignal2.transform.parent = body;
                        turnSignal2.transform.rotation = body.rotation;
                        turnSignal2.transform.localPosition = new Vector3(3.042803f, -0.257206f, 0f);
                        turnSignal2.transform.localScale = new Vector3(1f, 1f, 1f);
                        SpriteRenderer turnSignal2Renderer = turnSignal2.GetOrAddComponent<SpriteRenderer>();
                        turnSignal2Renderer.sprite = ModAPI.LoadSprite("Textures/Cars/Lada 2104/TurnSignal2.png");
                        turnSignal2Renderer.sortingOrder = body.GetComponent<SpriteRenderer>().sortingOrder + 1;
                        turnSignal2.AddComponent<Optout>();
                        ModAPI.CreateLight(turnSignal2.transform, new Color32(255, 146, 0, 255), 1f, 1f);

                        GameObject reversingLamp = new GameObject("Reversing Lamp");
                        reversingLamp.transform.parent = body;
                        reversingLamp.transform.rotation = body.rotation;
                        reversingLamp.transform.localPosition = new Vector3(-3.185709f, -0.271485f);
                        reversingLamp.transform.localScale = new Vector3(1f, 1f, 1f);
                        SpriteRenderer reversingLampRenderer = reversingLamp.GetOrAddComponent<SpriteRenderer>();
                        reversingLampRenderer.sprite = ModAPI.LoadSprite("Textures/Cars/Lada 2104/ReversingLamp.png");
                        reversingLampRenderer.sortingOrder = body.GetComponent<SpriteRenderer>().sortingOrder + 1;
                        reversingLamp.AddComponent<Optout>();
                        ModAPI.CreateLight(reversingLamp.transform, Color.white, 1f, 1f);

                        Transform bonnet = Instance.transform.GetChild(3);
                        bonnet.GetComponent<HingeJoint2D>().anchor = new Vector2(-0.87f, 0.08f);
                        if (firstSpawn)
                        {
                            bonnet.localPosition = new Vector3(2.44f, 0.015f);
                        }
                        bonnet.GetComponent<PolygonCollider2D>().SetPath(0, new Vector2[]
                        {
                            new Vector2(-0.8714285f, 0.1142857f),
                            new Vector2(-0.8721212f, 0.05846947f),
                            new Vector2(0.8714285f, -0.1142857f),
                            new Vector2(0.8703743f, -0.05867064f)
                        });

                        Instance.transform.GetChild(4).gameObject.SetActive(false);

                        Transform frontDoor = Instance.transform.GetChild(5);
                        if (firstSpawn)
                        {
                            frontDoor.localPosition = new Vector3(0.685f, 0.015f);
                        }
                        frontDoor.GetComponent<PolygonCollider2D>().SetPath(0, new Vector2[]
                        {
                            new Vector2(-0.8f, 0.8571429f),
                            new Vector2(-0.8f, -0.8571429f),
                            new Vector2(0.8f, -0.8571429f),
                            new Vector2(0.7972623f, -0.09603468f),
                            new Vector2(0.5215713f, 0.4979815f),
                            new Vector2(0.2152529f, 0.8591825f)
                        });

                        Transform backDoor = Instance.transform.GetChild(6);
                        if (firstSpawn)
                        {
                            backDoor.localPosition = new Vector3(-0.84f, 0.027f);
                        }
                        backDoor.GetComponent<PolygonCollider2D>().SetPath(0, new Vector2[]
                        {
                            new Vector2(-0.461466f, 0.8428571f),
                            new Vector2(-0.7285714f, 0.1471105f),
                            new Vector2(-0.7285714f, -0.1811957f),
                            new Vector2(-0.1169578f, -0.8428571f),
                            new Vector2(0.7285714f, -0.8428571f),
                            new Vector2(0.7285714f, 0.8428571f)
                        });

                        if (firstSpawn)
                        {
                            Instance.transform.GetChild(7).localPosition = new Vector3(2.3f, -0.35f);
                            Instance.transform.GetChild(8).localPosition = new Vector3(-2.1f, -0.35f);
                        }

                        if (firstSpawn)
                        {
                            body.GetComponent<RandomCarTextureBehaviour>().chosenIndex = UnityEngine.Random.Range(0, 5);
                        }
                        body.GetComponent<RandomCarTextureBehaviour>().Textures = new RandomCarTextureBehaviour.CarSprites[]
                        {
                            new RandomCarTextureBehaviour.CarSprites
                            {
                                Body = ModAPI.LoadSprite("Textures/Cars/Lada 2104/BlackColor/Body.png"),
                                Background = ModAPI.LoadSprite("Textures/Cars/Lada 2104/Background.png"),
                                Bonnet = ModAPI.LoadSprite("Textures/Cars/Lada 2104/BlackColor/Bonnet.png"),
                                FrontDoor = ModAPI.LoadSprite("Textures/Cars/Lada 2104/BlackColor/FrontDoor.png"),
                                BackDoor = ModAPI.LoadSprite("Textures/Cars/Lada 2104/BlackColor/BackDoor.png")
                            },
                            new RandomCarTextureBehaviour.CarSprites
                            {
                                Body = ModAPI.LoadSprite("Textures/Cars/Lada 2104/BlueColor/Body.png"),
                                Background = ModAPI.LoadSprite("Textures/Cars/Lada 2104/Background.png"),
                                Bonnet = ModAPI.LoadSprite("Textures/Cars/Lada 2104/BlueColor/Bonnet.png"),
                                FrontDoor = ModAPI.LoadSprite("Textures/Cars/Lada 2104/BlueColor/FrontDoor.png"),
                                BackDoor = ModAPI.LoadSprite("Textures/Cars/Lada 2104/BlueColor/BackDoor.png")
                            },
                            new RandomCarTextureBehaviour.CarSprites
                            {
                                Body = ModAPI.LoadSprite("Textures/Cars/Lada 2104/GrayColor/Body.png"),
                                Background = ModAPI.LoadSprite("Textures/Cars/Lada 2104/Background.png"),
                                Bonnet = ModAPI.LoadSprite("Textures/Cars/Lada 2104/GrayColor/Bonnet.png"),
                                FrontDoor = ModAPI.LoadSprite("Textures/Cars/Lada 2104/GrayColor/FrontDoor.png"),
                                BackDoor = ModAPI.LoadSprite("Textures/Cars/Lada 2104/GrayColor/BackDoor.png")
                            },
                            new RandomCarTextureBehaviour.CarSprites
                            {
                                Body = ModAPI.LoadSprite("Textures/Cars/Lada 2104/GreenColor/Body.png"),
                                Background = ModAPI.LoadSprite("Textures/Cars/Lada 2104/Background.png"),
                                Bonnet = ModAPI.LoadSprite("Textures/Cars/Lada 2104/GreenColor/Bonnet.png"),
                                FrontDoor = ModAPI.LoadSprite("Textures/Cars/Lada 2104/GreenColor/FrontDoor.png"),
                                BackDoor = ModAPI.LoadSprite("Textures/Cars/Lada 2104/GreenColor/BackDoor.png")
                            },
                            new RandomCarTextureBehaviour.CarSprites
                            {
                                Body = ModAPI.LoadSprite("Textures/Cars/Lada 2104/RedColor/Body.png"),
                                Background = ModAPI.LoadSprite("Textures/Cars/Lada 2104/Background.png"),
                                Bonnet = ModAPI.LoadSprite("Textures/Cars/Lada 2104/RedColor/Bonnet.png"),
                                FrontDoor = ModAPI.LoadSprite("Textures/Cars/Lada 2104/RedColor/FrontDoor.png"),
                                BackDoor = ModAPI.LoadSprite("Textures/Cars/Lada 2104/RedColor/BackDoor.png")
                            }
                        };

                        CarOpticsBehaviour carOptics = body.gameObject.GetOrAddComponent<CarOpticsBehaviour>();
                        carOptics.FailedEngineStartup = true;
                        carOptics.Headlights = new GameObject[]
                        {
                            headlight.gameObject,
                            taillight
                        };
                        carOptics.TurnSignals = new GameObject[]
                        {
                            turnSignal1,
                            turnSignal2
                        };
                        carOptics.ReversingLamp = reversingLamp;
                        carOptics.FailedEngineStartupSound = ModAPI.LoadSound("Sounds/FailedEngineStartup.mp3");

                        NoCollide noCollide = Instance.GetComponent<NoCollide>();
                        noCollide.NoCollideSetA = Instance.GetComponentsInChildren<Collider2D>();
                        noCollide.NoCollideSetB = Instance.GetComponentsInChildren<Collider2D>();
                    }
                }
            );
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Car"),
                    NameOverride = "Hummer H1" +tag,
                    NameToOrderByOverride = "!",
                    DescriptionOverride = "Four-wheel-drive utility vehicle based on the M998 Humvee, which was created by AM General. The vehicle was produced from 1992 through 2006, and was the first of what became the Hummer line. Originally designed strictly for military use, the off-road vehicle was released to the civilian market due to market demand.",
                    CategoryOverride = ModAPI.FindCategory("Vehicles"),
                    ThumbnailOverride = ModAPI.LoadSprite("Textures/Cars/Hummer H1/Thumb.png"), 
                    AfterSpawn = (Instance) =>
                    {
                        Transform body = Instance.transform.GetChild(0);
                        bool firstSpawn = body.localPosition == Vector3.zero;
                        if (firstSpawn)
                        {
                            body.GetComponent<CarBehaviour>().MotorSpeed = -3100f;
                        }
                        BoxCollider2D[] boxColliders = body.GetComponents<BoxCollider2D>();
                        boxColliders[0].offset = new Vector2(-0.4425528f, -0.6865115f);
                        boxColliders[0].size = new Vector2(0.7118573f, 0.3165189f);
                        boxColliders[1].offset = new Vector2(-0.8529415f, -0.03636563f);
                        boxColliders[1].size = new Vector2(0.1621748f, 1.11035f);
                        boxColliders[2].offset = new Vector2(0.8854706f, -0.6751637f);
                        boxColliders[2].size = new Vector2(0.68918f, 0.3494469f);
                        boxColliders[3].offset = new Vector2(0.5598401f, 0.06309009f);
                        boxColliders[3].size = new Vector2(0.2034953f, 1.221151f);
                        body.GetComponent<PolygonCollider2D>().offset = new Vector2(0f, 0.028357f);
                        body.GetComponent<PolygonCollider2D>().SetPath(0, new Vector2[]
                        {
                            new Vector2(2.142774f, 0.0433135f),
                            new Vector2(2.14286f, 0.2427346f),
                            new Vector2(2.028517f, 0.2427998f),
                            new Vector2(1.971389f, 1.014234f),
                            new Vector2(-0.8857394f, 1.014222f),
                            new Vector2(-0.9429132f, 0.9285183f),
                            new Vector2(-1.114345f, 0.2429981f),
                            new Vector2(-1.174045f, 0.2429996f),
                            new Vector2(-1.174348f, 0.009822398f),
                            new Vector2(-2.88577f, 0.009885341f),
                            new Vector2(-2.885741f, 0.2430139f),
                            new Vector2(-2.94285f, 0.2430294f),
                            new Vector2(-2.942976f, -0.6426549f),
                            new Vector2(-0.9429818f, -0.6429099f),
                            new Vector2(-0.6877806f, -1.071776f),
                            new Vector2(2.456642f, -1.071214f),
                            new Vector2(2.656974f, -0.6716815f),
                            new Vector2(4.056909f, -0.6715363f),
                            new Vector2(4.143048f, -0.5858107f),
                            new Vector2(4.143254f, -0.2142706f),
                            new Vector2(2.628627f, -0.214393f)
                        });
                        body.GetComponent<PolygonCollider2D>().SetPath(1, new Vector2[]
                        {
                            new Vector2(1.885841f, 0.8142986f),
                            new Vector2(1.885638f, -0.8429346f),
                            new Vector2(-0.1050253f, -0.8429241f),
                            new Vector2(-0.6381247f, -0.7111959f),
                            new Vector2(-0.911347f, 0.2348347f),
                            new Vector2(-0.742581f, 0.8141136f)
                        });
                        body.GetChild(0).localPosition = new Vector3(0.8f, -0.014292f);

                        WheelJoint2D[] wheels = body.GetComponents<WheelJoint2D>();
                        wheels[0].anchor = new Vector2(3.366f, -1.0f);
                        wheels[0].connectedBody.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("Textures/Cars/Hummer H1/Wheel.png");
                        wheels[0].connectedBody.GetComponent<CircleCollider2D>().radius = 0.6857143f;
                        wheels[1].anchor = new Vector2(-1.711f, -1.0f);
                        wheels[1].connectedBody.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("Textures/Cars/Hummer H1/Wheel.png");
                        wheels[1].connectedBody.GetComponent<CircleCollider2D>().radius = 0.6857143f;

                        if (firstSpawn)
                        {
                            wheels[0].connectedBody.transform.localPosition = new Vector3(3.366f, -1.0f);
                            wheels[1].connectedBody.transform.localPosition = new Vector3(-1.711f, -1.0f);
                        }

                        LightSprite taillight = ModAPI.CreateLight(body, Color.red, 3.5f, 1.5f);
                        taillight.transform.localPosition = new Vector3(-2.934f, -0.112f);

                        LightSprite reversingLamp = ModAPI.CreateLight(body, Color.white, 1f, 1f);
                        reversingLamp.transform.localPosition = new Vector3(-2.933f, -0.139f);

                        Transform bonnet = Instance.transform.GetChild(3);
                        bonnet.GetComponent<SpriteRenderer>().sortingOrder = wheels[0].connectedBody.GetComponent<SpriteRenderer>().sortingOrder + 1;
                        bonnet.GetComponent<FixedJoint2D>().breakForce = 300f;
                        bonnet.GetComponent<HingeJoint2D>().anchor = new Vector2(0.9283743f, -0.3900707f);
                        bonnet.GetComponent<HingeJoint2D>().limits = new JointAngleLimits2D
                        {
                            min = 0,
                            max = 55
                        };
                        if (firstSpawn)
                        {
                            bonnet.localPosition = new Vector3(3.12852f, -0.128623f);
                        }
                        bonnet.GetComponent<PolygonCollider2D>().SetPath(0, new Vector2[]
                        {
                            new Vector2(-0.5345829f, 0.4f),
                            new Vector2(-1.014286f, 0.4f),
                            new Vector2(-1.012003f, 0.1802728f),
                            new Vector2(-0.3898644f, -0.1411351f),
                            new Vector2(-0.2047734f, -0.1415623f),
                            new Vector2(-0.07092929f, -0.08663893f),
                            new Vector2(0.545867f, -0.08511019f),
                            new Vector2(0.8657677f, -0.4001054f),
                            new Vector2(1.014286f, -0.4f),
                            new Vector2(1.014286f, -0.06328857f),
                            new Vector2(0.8967597f, 0.07804364f)
                        });

                        LightSprite headlight = ModAPI.CreateLight(bonnet, Color.white, 3.5f, 1.5f);
                        headlight.transform.localPosition = new Vector3(0.9874802f, -0.217f);

                        GameObject turnSignal = new GameObject("Turn Signal");
                        turnSignal.transform.parent = bonnet;
                        turnSignal.transform.rotation = bonnet.rotation;
                        turnSignal.transform.localPosition = new Vector3(0.800002f, -0.071431f, 0f);
                        turnSignal.transform.localScale = new Vector3(1f, 1f, 1f);
                        SpriteRenderer turnSignalRenderer = turnSignal.AddComponent<SpriteRenderer>();
                        turnSignalRenderer.sprite = ModAPI.LoadSprite("Textures/Cars/Hummer H1/TurnSignal.png");
                        turnSignalRenderer.sortingOrder = bonnet.GetComponent<SpriteRenderer>().sortingOrder + 1;
                        turnSignal.AddComponent<Optout>();
                        ModAPI.CreateLight(turnSignal.transform, new Color32(255, 146, 0, 255), 1f, 1f);

                        bonnet.GetComponent<JointBreakSoundBehaviour>().OnBreak.AddListener(() =>
                        {
                            List<GameObject> headlightList = body.GetComponent<CarOpticsBehaviour>().Headlights.ToList();
                            headlightList.Remove(headlight.gameObject);
                            body.GetComponent<CarOpticsBehaviour>().Headlights = headlightList.ToArray();
                            body.GetComponent<CarOpticsBehaviour>().TurnSignals = new GameObject[0];
                            ModAPI.CreateParticleEffect("BrokenElectronicsSpark", bonnet.position + new Vector3(0.9283743f, -0.3900707f));
                            headlight.gameObject.SetActive(false);
                            turnSignal.SetActive(false);
                        });

                        Instance.transform.GetChild(4).gameObject.SetActive(false);

                        Transform frontDoor = Instance.transform.GetChild(5);
                        if (firstSpawn)
                        {
                            frontDoor.localPosition = new Vector3(1.24286f, 0.0142914f);
                        }
                        UnityEngine.Object.Destroy(frontDoor.GetComponent<PolygonCollider2D>());
                        frontDoor.gameObject.AddComponent<BoxCollider2D>().size = new Vector2(1.342857f, 1.714286f);

                        Transform backDoor = Instance.transform.GetChild(6);
                        if (firstSpawn)
                        {
                            backDoor.localPosition = new Vector3(-0.2142861f, 0.0142914f);
                        }
                        backDoor.GetComponent<PolygonCollider2D>().SetPath(0, new Vector2[]
                        {
                            new Vector2(-0.4999099f, 0.8571624f),
                            new Vector2(-0.7284815f, 0.2569818f),
                            new Vector2(-0.7285628f, -4.300475e-05f),
                            new Vector2(-0.2143337f, -0.8571276f),
                            new Vector2(0.7285714f, -0.8571429f),
                            new Vector2(0.7285714f, 0.8571429f)
                        });

                        if (firstSpawn)
                        {
                            Transform engine = Instance.transform.GetChild(7);
                            engine.localScale = new Vector3(0.9f, 0.9f);
                            engine.localPosition = new Vector3(3.355f, -0.283f);

                            Instance.transform.GetChild(8).localPosition = new Vector3(-2.144f, -0.285f);
                        }

                        body.GetComponent<RandomCarTextureBehaviour>().Textures = new RandomCarTextureBehaviour.CarSprites[] 
                        {
                            new RandomCarTextureBehaviour.CarSprites
                            {
                                Body = ModAPI.LoadSprite("Textures/Cars/Hummer H1/BlackColor/Body.png"),
                                Background = ModAPI.LoadSprite("Textures/Cars/Hummer H1/Background.png"),
                                Bonnet = ModAPI.LoadSprite("Textures/Cars/Hummer H1/BlackColor/Bonnet.png"),
                                FrontDoor = ModAPI.LoadSprite("Textures/Cars/Hummer H1/BlackColor/FrontDoor.png"),
                                BackDoor = ModAPI.LoadSprite("Textures/Cars/Hummer H1/BlackColor/BackDoor.png")
                            },
                            new RandomCarTextureBehaviour.CarSprites
                            {
                                Body = ModAPI.LoadSprite("Textures/Cars/Hummer H1/GrayColor/Body.png"),
                                Background = ModAPI.LoadSprite("Textures/Cars/Hummer H1/Background.png"),
                                Bonnet = ModAPI.LoadSprite("Textures/Cars/Hummer H1/GrayColor/Bonnet.png"),
                                FrontDoor = ModAPI.LoadSprite("Textures/Cars/Hummer H1/GrayColor/FrontDoor.png"),
                                BackDoor = ModAPI.LoadSprite("Textures/Cars/Hummer H1/GrayColor/BackDoor.png")
                            },
                            new RandomCarTextureBehaviour.CarSprites
                            {
                                Body = ModAPI.LoadSprite("Textures/Cars/Hummer H1/RedColor/Body.png"),
                                Background = ModAPI.LoadSprite("Textures/Cars/Hummer H1/Background.png"),
                                Bonnet = ModAPI.LoadSprite("Textures/Cars/Hummer H1/RedColor/Bonnet.png"),
                                FrontDoor = ModAPI.LoadSprite("Textures/Cars/Hummer H1/RedColor/FrontDoor.png"),
                                BackDoor = ModAPI.LoadSprite("Textures/Cars/Hummer H1/RedColor/BackDoor.png")
                            },
                            new RandomCarTextureBehaviour.CarSprites
                            {
                                Body = ModAPI.LoadSprite("Textures/Cars/Hummer H1/YellowColor/Body.png"),
                                Background = ModAPI.LoadSprite("Textures/Cars/Hummer H1/Background.png"),
                                Bonnet = ModAPI.LoadSprite("Textures/Cars/Hummer H1/YellowColor/Bonnet.png"),
                                FrontDoor = ModAPI.LoadSprite("Textures/Cars/Hummer H1/YellowColor/FrontDoor.png"),
                                BackDoor = ModAPI.LoadSprite("Textures/Cars/Hummer H1/YellowColor/BackDoor.png")
                            }
                        };

                        CarOpticsBehaviour carOptics = body.gameObject.GetOrAddComponent<CarOpticsBehaviour>();
                        carOptics.Headlights = new GameObject[]
                        {
                            headlight.gameObject,
                            taillight.gameObject
                        };
                        carOptics.TurnSignals = new GameObject[]
                        {
                            turnSignal
                        };
                        carOptics.ReversingLamp = reversingLamp.gameObject;

                        NoCollide noCollide = Instance.GetComponent<NoCollide>();
                        noCollide.NoCollideSetA = Instance.GetComponentsInChildren<Collider2D>();
                        noCollide.NoCollideSetB = Instance.GetComponentsInChildren<Collider2D>();
                    }
                }
            );
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Car"),
                    NameOverride = "Hummer H1 Military" +tag,
                    NameToOrderByOverride = "!",
                    DescriptionOverride = "Hummer H1 but military.",
                    CategoryOverride = ModAPI.FindCategory("Vehicles"),
                    ThumbnailOverride = ModAPI.LoadSprite("Textures/Cars/Hummer H1/Thumb2.png"), 
                    AfterSpawn = (Instance) =>
                    {
                        Transform body = Instance.transform.GetChild(0);
                        bool firstSpawn = body.localPosition == Vector3.zero;
                        if (firstSpawn)
                        {
                            body.GetComponent<RandomCarTextureBehaviour>().chosenIndex = UnityEngine.Random.Range(0, 2);
                            body.GetComponent<CarBehaviour>().MotorSpeed = -3050f;
                        }
                        BoxCollider2D[] boxColliders = body.GetComponents<BoxCollider2D>();
                        boxColliders[0].offset = new Vector2(0.8932201f, -0.7014027f);
                        boxColliders[0].size = new Vector2(0.6993212f, 0.3226223f);
                        boxColliders[1].offset = new Vector2(0.5522f, -0.02798796f);
                        boxColliders[1].size = new Vector2(0.2182359f, 1.335423f);
                        boxColliders[2].offset = new Vector2(-0.4047221f, -0.7252221f);
                        boxColliders[2].size = new Vector2(0.6617968f, 0.3002539f);
                        boxColliders[3].offset = new Vector2(-0.812965f, -0.04964495f);
                        boxColliders[3].size = new Vector2(0.2242718f, 1.10992f);
                        body.GetComponent<PolygonCollider2D>().SetPath(0, new Vector2[]
                        {
                            new Vector2(2.142774f, 0.0433135f),
                            new Vector2(2.14286f, 0.2427346f),
                            new Vector2(2.028517f, 0.2427998f),
                            new Vector2(1.971389f, 1.014234f),
                            new Vector2(1.342682f, 1.014179f),
                            new Vector2(1.342811f, 1.071436f),
                            new Vector2(0.771365f, 1.071429f),
                            new Vector2(0.7712709f, 0.8142624f),
                            new Vector2(1.885841f, 0.8142986f),
                            new Vector2(1.885638f, -0.8429346f),
                            new Vector2(-0.1050253f, -0.8429241f),
                            new Vector2(-0.6381247f, -0.7111959f),
                            new Vector2(-0.911347f, 0.2348347f),
                            new Vector2(-0.742581f, 0.8141136f),
                            new Vector2(-0.113258f, 0.8142576f),
                            new Vector2(-0.1134357f, 1.071426f),
                            new Vector2(-0.2857676f, 1.071425f),
                            new Vector2(-0.2858009f, 1.014233f),
                            new Vector2(-0.8857394f, 1.014222f),
                            new Vector2(-0.9429132f, 0.9285183f),
                            new Vector2(-1.037319f, 0.2142334f),
                            new Vector2(-2.942794f, 0.2141121f),
                            new Vector2(-2.942976f, -0.6426549f),
                            new Vector2(-0.9429818f, -0.6429098f),
                            new Vector2(-0.6877806f, -1.071776f),
                            new Vector2(2.456642f, -1.071214f),
                            new Vector2(2.656974f, -0.6716815f),
                            new Vector2(4.056909f, -0.6715363f),
                            new Vector2(4.143048f, -0.5858107f),
                            new Vector2(4.143254f, -0.2142706f),
                            new Vector2(2.628627f, -0.214393f)
                        });
                        body.GetComponent<PolygonCollider2D>().pathCount = 1;
                        PhysicalProperties properties = UnityEngine.Object.Instantiate(body.GetComponent<PhysicalBehaviour>().Properties);
                        properties.BulletSpeedAbsorptionPower = 6f;
                        body.GetComponent<PhysicalBehaviour>().Properties = properties;
                        body.GetChild(0).localPosition = new Vector3(0.8f, -0.04286f);

                        WheelJoint2D[] wheels = body.GetComponents<WheelJoint2D>();
                        wheels[0].anchor = new Vector2(3.366f, -1.03f);
                        wheels[0].connectedBody.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("Textures/Cars/Hummer H1/Wheel.png");
                        wheels[0].connectedBody.GetComponent<CircleCollider2D>().radius = 0.6857143f;
                        wheels[1].anchor = new Vector2(-1.711f, -1.03f);
                        wheels[1].connectedBody.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("Textures/Cars/Hummer H1/Wheel.png");
                        wheels[1].connectedBody.GetComponent<CircleCollider2D>().radius = 0.6857143f;

                        if (firstSpawn)
                        {
                            wheels[0].connectedBody.transform.localPosition = new Vector3(3.366f, -1.03f);
                            wheels[1].connectedBody.transform.localPosition = new Vector3(-1.711f, -1.03f);
                        }

                        LightSprite taillight = ModAPI.CreateLight(body, Color.red, 3.5f, 1.5f);
                        taillight.transform.localPosition = new Vector3(-2.934f, -0.112f);

                        LightSprite reversingLamp = ModAPI.CreateLight(body, Color.white, 1f, 1f);
                        reversingLamp.transform.localPosition = new Vector3(-2.933f, -0.139f);

                        Transform bonnet = Instance.transform.GetChild(3);
                        bonnet.GetComponent<SpriteRenderer>().sortingOrder = body.GetComponent<SpriteRenderer>().sortingOrder + 1;
                        bonnet.GetComponent<FixedJoint2D>().breakForce = 350f;
                        bonnet.GetComponent<HingeJoint2D>().anchor = new Vector2(0.9283743f, -0.3900707f);
                        bonnet.GetComponent<HingeJoint2D>().limits = new JointAngleLimits2D
                        {
                            min = 0,
                            max = 55
                        };
                        if (firstSpawn)
                        {
                            bonnet.localPosition = new Vector3(3.12852f, -0.157142f);
                        }
                        bonnet.GetComponent<PolygonCollider2D>().SetPath(0, new Vector2[]
                        {
                            new Vector2(-0.5345829f, 0.4f),
                            new Vector2(-1.014286f, 0.4f),
                            new Vector2(-1.012003f, 0.1802728f),
                            new Vector2(-0.3898644f, -0.1411351f),
                            new Vector2(-0.2047734f, -0.1415623f),
                            new Vector2(-0.07092929f, -0.08663893f),
                            new Vector2(0.545867f, -0.08511019f),
                            new Vector2(0.8657677f, -0.4001054f),
                            new Vector2(1.014286f, -0.4f),
                            new Vector2(1.014286f, -0.06328857f),
                            new Vector2(0.8967597f, 0.07804364f)
                        });

                        LightSprite headlight = ModAPI.CreateLight(bonnet, Color.white, 3.5f, 1.5f);
                        headlight.transform.localPosition = new Vector3(0.9874802f, -0.217f);

                        GameObject turnSignal = new GameObject("Turn Signal");
                        turnSignal.transform.parent = bonnet;
                        turnSignal.transform.rotation = bonnet.rotation;
                        turnSignal.transform.localPosition = new Vector3(0.800002f, -0.071431f, 0f);
                        turnSignal.transform.localScale = new Vector3(1f, 1f, 1f);
                        SpriteRenderer turnSignalRenderer = turnSignal.GetOrAddComponent<SpriteRenderer>();
                        turnSignalRenderer.sprite = ModAPI.LoadSprite("Textures/Cars/Hummer H1/TurnSignal.png");
                        turnSignalRenderer.sortingOrder = bonnet.GetComponent<SpriteRenderer>().sortingOrder + 1;
                        turnSignal.AddComponent<Optout>();
                        ModAPI.CreateLight(turnSignal.transform, new Color32(255, 146, 0, 255), 1f, 1f);

                        bonnet.GetComponent<JointBreakSoundBehaviour>().OnBreak.AddListener(() =>
                        {
                            List<GameObject> headlightList = body.gameObject.GetComponent<CarOpticsBehaviour>().Headlights.ToList();
                            headlightList.Remove(headlight.gameObject);
                            body.gameObject.GetComponent<CarOpticsBehaviour>().Headlights = headlightList.ToArray();
                            body.gameObject.GetComponent<CarOpticsBehaviour>().TurnSignals = new GameObject[0];
                            ModAPI.CreateParticleEffect("BrokenElectronicsSpark", bonnet.position + new Vector3(0.9283743f, -0.3900707f));
                            headlight.gameObject.SetActive(false);
                            turnSignal.SetActive(false);
                        });

                        Transform boot = Instance.transform.GetChild(4);
                        UnityEngine.Object.Destroy(boot.GetComponent<BoxCollider2D>());
                        boot.GetComponent<HingeJoint2D>().anchor = new Vector2(0.9999198f, 0.3283523f);
                        if (firstSpawn)
                        {
                            boot.localPosition = new Vector3(-1.942854f, 0.5714235f);
                        }
                        boot.GetComponent<PolygonCollider2D>().SetPath(0, new Vector2[]
                        {
                            new Vector2(1f, 0.3571429f),
                            new Vector2(0.8580877f, 0.3561639f),
                            new Vector2(-0.8859256f, -0.08020445f),
                            new Vector2(-1f, -0.3571429f),
                            new Vector2(-0.9437303f, -0.3571429f),
                            new Vector2(-0.8207295f, -0.1145015f),
                            new Vector2(0.9999917f, 0.2995541f)
                        });

                        Transform frontDoor = Instance.transform.GetChild(5);
                        if (firstSpawn)
                        {
                            frontDoor.localPosition = new Vector3(1.24286f, -0.014338f);
                        }
                        UnityEngine.Object.Destroy(frontDoor.GetComponent<PolygonCollider2D>());
                        frontDoor.gameObject.AddComponent<BoxCollider2D>().size = new Vector2(1.342857f, 1.714286f);

                        Transform backDoor = Instance.transform.GetChild(6);
                        if (firstSpawn)
                        {
                            backDoor.localPosition = new Vector3(-0.2142861f, -0.014338f);
                        }
                        backDoor.GetComponent<PolygonCollider2D>().SetPath(0, new Vector2[]
                        {
                            new Vector2(-0.4999099f, 0.8571624f),
                            new Vector2(-0.7284815f, 0.2569818f),
                            new Vector2(-0.7285628f, -4.300475e-05f),
                            new Vector2(-0.2143337f, -0.8571276f),
                            new Vector2(0.7285714f, -0.8571429f),
                            new Vector2(0.7285714f, 0.8571429f)
                        });

                        if (firstSpawn)
                        {
                            Transform engine = Instance.transform.GetChild(7);
                            engine.localScale = new Vector3(0.9f, 0.9f);
                            engine.localPosition = new Vector3(3.355f, -0.316f);

                            Instance.transform.GetChild(8).localPosition = new Vector3(-2.144f, -0.231f);
                        }

                        Sprite[] hatchSprites = new Sprite[]
                        {
                            ModAPI.LoadSprite("Textures/Cars/Hummer H1/MilitaryDesertSkin/Hatch.png"),
                            ModAPI.LoadSprite("Textures/Cars/Hummer H1/MilitarySkin/Hatch.png")
                        };
                        if (firstSpawn)
                        {
                            GameObject hatch = new GameObject("Hatch");
                            hatch.transform.parent = Instance.transform;
                            hatch.transform.rotation = body.transform.rotation;
                            hatch.transform.localScale = Vector3.one;
                            hatch.transform.localPosition = new Vector3(0.328508f, 1.057143f);
                            hatch.AddComponent<Optout>();
                            Instance.AddComponent<ObjectSerialisationHelper>().SerialisationObject = hatch;
                        }
                        ObjectSerialisationHelper hatchSerialisationHelper = Instance.GetComponent<ObjectSerialisationHelper>();
                        hatchSerialisationHelper.InstantiateOverride = (gameObject) =>
                        {
                            SpriteRenderer gameObjectRenderer = gameObject.AddComponent<SpriteRenderer>();
                            gameObjectRenderer.sortingOrder = boot.GetComponent<SpriteRenderer>().sortingOrder;
                            gameObjectRenderer.sortingLayerName = boot.GetComponent<SpriteRenderer>().sortingLayerName;
                            gameObject.AddComponent<RandomCarTextureChildBehaviour>().Sprites = hatchSprites;
                            gameObject.layer = LayerMask.NameToLayer("Objects");
                            BoxCollider2D boxCollider1 = gameObject.AddComponent<BoxCollider2D>();
                            boxCollider1.offset = new Vector2(0f, -0.02859946f);
                            boxCollider1.size = new Vector2(0.8857143f, 0.08565822f);
                            BoxCollider2D boxCollider2 = gameObject.AddComponent<BoxCollider2D>();
                            boxCollider2.offset = new Vector2(-3.321469e-05f, 0.02843906f);
                            boxCollider2.size = new Vector2(0.1426883f, 0.08597901f);
                            gameObject.AddComponent<Rigidbody2D>().mass = 0.3f;
                            FixedJoint2D fixedJoint = gameObject.AddComponent<FixedJoint2D>();
                            fixedJoint.connectedBody = body.GetComponent<Rigidbody2D>();
                            fixedJoint.breakForce = boot.GetComponent<FixedJoint2D>().breakForce;
                            Vector3 eulerAngles = gameObject.transform.eulerAngles;
                            gameObject.transform.eulerAngles = body.eulerAngles;
                            HingeJoint2D hingeJoint = gameObject.AddComponent<HingeJoint2D>();
                            hingeJoint.connectedBody = body.GetComponent<Rigidbody2D>();
                            hingeJoint.anchor = new Vector2(-0.4428571f, -0.07142854f);
                            hingeJoint.breakForce = 900f;
                            hingeJoint.breakTorque = 900f;
                            hingeJoint.limits = new JointAngleLimits2D()
                            {
                                min = -95,
                                max = 0
                            };
                            hingeJoint.useLimits = true;
                            gameObject.transform.eulerAngles = eulerAngles;
                            AudioSource audioSource = gameObject.AddComponent<AudioSource>();
                            audioSource.spread = 0f;
                            audioSource.volume = 1f;
                            audioSource.minDistance = 1f;
                            audioSource.spatialBlend = 1f;
                            audioSource.dopplerLevel = 0f;
                            gameObject.AddComponent<JointSerialisationHelper>().Joint = fixedJoint;
                            gameObject.AddComponent<JointSerialisationHelper>().Joint = hingeJoint;
                            JointBreakSoundBehaviour jointBreakSound = gameObject.AddComponent<JointBreakSoundBehaviour>();
                            jointBreakSound.Clips = boot.GetComponent<JointBreakSoundBehaviour>().Clips;
                            jointBreakSound.AudioSource = audioSource;
                            jointBreakSound.Joints = new Joint2D[]
                            {
                                hingeJoint
                            };
                            jointBreakSound.OnBreak = new UnityEvent();
                            jointBreakSound.OnBreak.AddListener(() =>
                            {
                                jointBreakSound.Joints = new Joint2D[0];
                            });
                            gameObject.AddComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Metal");
                            gameObject.AddComponent<AudioSourceTimeScaleBehaviour>();

                            gameObject.AddComponent<Optout>();

                            NoCollide noCollide = Instance.GetComponent<NoCollide>();
                            noCollide.NoCollideSetA = Instance.GetComponentsInChildren<Collider2D>();
                            noCollide.NoCollideSetB = Instance.GetComponentsInChildren<Collider2D>();
                        };
                        if (firstSpawn)
                        {
                            hatchSerialisationHelper.InstantiateOverride(hatchSerialisationHelper.SerialisationObject);
                        }

                        body.GetComponent<RandomCarTextureBehaviour>().Textures = new RandomCarTextureBehaviour.CarSprites[]
                        {
                            new RandomCarTextureBehaviour.CarSprites
                            {
                                Body = ModAPI.LoadSprite("Textures/Cars/Hummer H1/MilitaryDesertSkin/Body.png"),
                                Background = ModAPI.LoadSprite("Textures/Cars/Hummer H1/MilitaryDesertSkin/Background.png"),
                                Bonnet = ModAPI.LoadSprite("Textures/Cars/Hummer H1/MilitaryDesertSkin/Bonnet.png"),
                                Boot = ModAPI.LoadSprite("Textures/Cars/Hummer H1/MilitaryDesertSkin/Boot.png"),
                                FrontDoor = ModAPI.LoadSprite("Textures/Cars/Hummer H1/MilitaryDesertSkin/FrontDoor.png"),
                                BackDoor = ModAPI.LoadSprite("Textures/Cars/Hummer H1/MilitaryDesertSkin/BackDoor.png")
                            },
                            new RandomCarTextureBehaviour.CarSprites
                            {
                                Body = ModAPI.LoadSprite("Textures/Cars/Hummer H1/MilitarySkin/Body.png"),
                                Background = ModAPI.LoadSprite("Textures/Cars/Hummer H1/MilitarySkin/Background.png"),
                                Bonnet = ModAPI.LoadSprite("Textures/Cars/Hummer H1/MilitarySkin/Bonnet.png"),
                                Boot = ModAPI.LoadSprite("Textures/Cars/Hummer H1/MilitarySkin/Boot.png"),
                                FrontDoor = ModAPI.LoadSprite("Textures/Cars/Hummer H1/MilitarySkin/FrontDoor.png"),
                                BackDoor = ModAPI.LoadSprite("Textures/Cars/Hummer H1/MilitarySkin/BackDoor.png")
                            }
                        };

                        CarOpticsBehaviour carOptics = body.gameObject.GetOrAddComponent<CarOpticsBehaviour>();
                        carOptics.Headlights = new GameObject[]
                        {
                            headlight.gameObject,
                            taillight.gameObject
                        };
                        carOptics.TurnSignals = new GameObject[]
                        {
                            turnSignal
                        };
                        carOptics.ReversingLamp = reversingLamp.gameObject;
                    }
                }
            );
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Car"),
                    NameOverride = "BMW M5 F90" +tag,
                    NameToOrderByOverride = "!",
                    DescriptionOverride = "",
                    CategoryOverride = ModAPI.FindCategory("Vehicles"),
                    ThumbnailOverride = ModAPI.LoadSprite("Textures/Cars/BMW M5 F90/Thumb.png"), 
                    AfterSpawn = (Instance) =>
                    {
                        Transform body = Instance.transform.GetChild(0);
                        bool firstSpawn = body.localPosition == Vector3.zero;
                        if (firstSpawn)
                        {
                            body.GetComponent<CarBehaviour>().MotorSpeed = -4850f;
                        }
                        BoxCollider2D[] boxColliders = body.GetComponents<BoxCollider2D>();
                        boxColliders[0].offset = new Vector2(0.0185051f, -0.6445808f);
                        boxColliders[0].size = new Vector2(0.7351952f, 0.3980198f);
                        boxColliders[1].offset = new Vector2(-0.2947388f, 0.125545f);
                        boxColliders[1].size = new Vector2(0.2218513f, 1.21115f);
                        boxColliders[2].offset = new Vector2(-1.269417f, -0.6495647f);
                        boxColliders[2].size = new Vector2(0.6644325f, 0.3261127f);
                        boxColliders[3].offset = new Vector2(-1.696531f, 0.01295042f);
                        boxColliders[3].size = new Vector2(0.2514458f, 1.129582f);
                        body.GetComponent<PolygonCollider2D>().SetPath(0, new Vector2[]
                        {
                            new Vector2(3.542883f, -0.8288047f),
                            new Vector2(3.542857f, -0.4048213f),
                            new Vector2(3.428571f, -0.03311843f),
                            new Vector2(1.514205f, 0.2292352f),
                            new Vector2(1.514286f, 0.2824824f),
                            new Vector2(1.360908f, 0.2830796f),
                            new Vector2(1.106244f, 0.4834788f),
                            new Vector2(0.3950077f, 0.8772881f),
                            new Vector2(-0.3152425f, 0.9717709f),
                            new Vector2(-1.191791f, 0.9711298f),
                            new Vector2(-1.743479f, 0.8534529f),
                            new Vector2(-2.542146f, 0.5141158f),
                            new Vector2(-2.635382f, -0.03078544f),
                            new Vector2(-3.399124f, -0.03073749f),
                            new Vector2(-3.54007f, -0.3404343f),
                            new Vector2(-3.403266f, -0.7696168f),
                            new Vector2(-2.723561f, -0.9389736f),
                            new Vector2(3.171155f, -0.9435956f)
                        });
                        body.GetComponent<PolygonCollider2D>().SetPath(1, new Vector2[]
                        {
                            new Vector2(-2.00018f, 0.485719f),
                            new Vector2(-1.9999f, 0.5697989f),
                            new Vector2(-1.417173f, 0.8569055f),
                            new Vector2(-0.1492898f, 0.8572279f),
                            new Vector2(0.3262769f, 0.7524905f),
                            new Vector2(1.426542f, 0.1684611f),
                            new Vector2(1.428516f, -0.8000567f),
                            new Vector2(-1.255342f, -0.7974393f)
                        });
                        body.GetChild(0).localPosition = new Vector3(0.199997f, -0.042856f);

                        WheelJoint2D[] wheels = body.GetComponents<WheelJoint2D>();
                        wheels[0].anchor = new Vector2(2.46f, -0.793f);
                        wheels[1].anchor = new Vector2(-2.063f, -0.793f);

                        if (firstSpawn)
                        {
                            wheels[0].connectedBody.transform.localPosition = new Vector3(2.46f, -0.793f);
                            wheels[1].connectedBody.transform.localPosition = new Vector3(-2.063f, -0.793f);
                        }

                        GameObject headlight = new GameObject("Headlight");
                        headlight.transform.parent = body;
                        headlight.transform.rotation = body.rotation;
                        headlight.transform.localPosition = new Vector3(3.214276f, -0.1571992f);
                        headlight.transform.localScale = new Vector3(1f, 1f);
                        SpriteRenderer headlightRenderer = headlight.GetOrAddComponent<SpriteRenderer>();
                        headlightRenderer.sprite = ModAPI.LoadSprite("Textures/Cars/BMW M5 F90/Headlight.png");
                        headlightRenderer.sortingOrder = body.GetComponent<SpriteRenderer>().sortingOrder + 1;
                        headlight.AddComponent<Optout>();
                        ModAPI.CreateLight(headlight.transform, Color.white, 3.5f, 1.5f);

                        GameObject taillight = new GameObject("Taillight");
                        taillight.transform.parent = body;
                        taillight.transform.rotation = body.rotation;
                        taillight.transform.localPosition = new Vector3(-3.185709f, 0.1142304f);
                        taillight.transform.localScale = new Vector3(1f, 1f);
                        SpriteRenderer taillightRenderer = taillight.GetOrAddComponent<SpriteRenderer>();
                        taillightRenderer.sprite = ModAPI.LoadSprite("Textures/Cars/BMW M5 F90/Taillight.png");
                        taillightRenderer.sortingOrder = body.GetComponent<SpriteRenderer>().sortingOrder + 1;
                        taillight.AddComponent<Optout>();
                        ModAPI.CreateLight(taillight.transform, Color.red, 3.5f, 1.5f);

                        LightSprite turnSignal1 = ModAPI.CreateLight(body, new Color32(255, 146, 0, 255), 1f, 1f);
                        turnSignal1.transform.localPosition = new Vector3(3.271f, -0.125f);

                        LightSprite turnSignal2 = ModAPI.CreateLight(body, new Color32(255, 146, 0, 255), 1f, 1f);
                        turnSignal2.transform.localPosition = new Vector3(-3.26f, 0.131f);

                        LightSprite reversingLamp = ModAPI.CreateLight(body, Color.white, 1f, 1f);
                        reversingLamp.transform.localPosition = new Vector3(-3.386f, 0.127f);

                        Transform bonnet = Instance.transform.GetChild(3);
                        bonnet.GetComponent<HingeJoint2D>().anchor = new Vector2(-0.9428809f, 0.1287231f);
                        if (firstSpawn)
                        {
                            bonnet.localPosition = new Vector3(2.49999f, 0.1000027f);
                        }
                        bonnet.GetComponent<PolygonCollider2D>().SetPath(0, new Vector2[]
                        {
                            new Vector2(-0.9571429f, 0.1857143f),
                            new Vector2(-0.9562626f, 0.13167f),
                            new Vector2(0.6687631f, -0.1034289f),
                            new Vector2(0.8142713f, -0.1857143f),
                            new Vector2(0.9571429f, -0.1857143f),
                            new Vector2(0.6998358f, -0.01441953f),
                            new Vector2(-0.7857933f, 0.1856971f)
                        });

                        Transform boot = Instance.transform.GetChild(4);
                        boot.GetComponent<HingeJoint2D>().anchor = new Vector2(0.4681487f, 0.2259247f);
                        if (firstSpawn)
                        {
                            boot.localPosition = new Vector3(-3.01435f, 0.24296f);
                        }
                        boot.GetComponent<BoxCollider2D>().offset = new Vector2(-0.4118117f, -0.1446873f);
                        boot.GetComponent<BoxCollider2D>().size = new Vector2(0.1192338f, 0.08299851f);
                        boot.GetComponent<PolygonCollider2D>().SetPath(0, new Vector2[]
                        {
                            new Vector2(-0.3856234f, 0.1279374f),
                            new Vector2(-0.385169f, -0.2713265f),
                            new Vector2(-0.3287662f, -0.2714205f),
                            new Vector2(-0.3285711f, 0.02907726f),
                            new Vector2(0.4410973f, 0.1574926f),
                            new Vector2(0.4714286f, 0.2714286f),
                            new Vector2(0.01417835f, 0.1287098f)
                        });

                        Transform frontDoor = Instance.transform.GetChild(5);
                        if (firstSpawn)
                        {
                            frontDoor.localPosition = new Vector3(0.514286f, 0.0285201f);
                        }
                        frontDoor.GetComponent<PolygonCollider2D>().SetPath(0, new Vector2[]
                        {
                            new Vector2(-0.9428571f, 0.8571429f),
                            new Vector2(-0.6846544f, 0.017946f),
                            new Vector2(-0.685322f, -0.8571743f),
                            new Vector2(0.9428571f, -0.8571429f),
                            new Vector2(0.9428571f, 0.04125424f),
                            new Vector2(-0.1266136f, 0.7428083f),
                            new Vector2(-0.6497065f, 0.8571429f)
                        });

                        Transform backDoor = Instance.transform.GetChild(6);
                        if (firstSpawn)
                        {
                            backDoor.localPosition = new Vector3(-1.114284f, 0.0285201f);
                        }
                        backDoor.GetComponent<PolygonCollider2D>().SetPath(0, new Vector2[]
                        {
                            new Vector2(-0.02858108f, 0.8571619f),
                            new Vector2(-0.9141942f, 0.5710715f),
                            new Vector2(-0.9142857f, 0.4374049f),
                            new Vector2(-0.08489598f, -0.8571429f),
                            new Vector2(0.9142857f, -0.8571429f),
                            new Vector2(0.9142857f, -0.1308719f),
                            new Vector2(0.6570932f, 0.8571381f)
                        });

                        if (firstSpawn)
                        {
                            Instance.transform.GetChild(7).localPosition = new Vector3(2.2f, -0.25f);
                            Instance.transform.GetChild(8).localPosition = new Vector3(-2.3f, -0.3f);
                        }

                        if (firstSpawn)
                        {
                            body.GetComponent<RandomCarTextureBehaviour>().chosenIndex = UnityEngine.Random.Range(0, 5);
                        }
                        body.GetComponent<RandomCarTextureBehaviour>().Textures = new RandomCarTextureBehaviour.CarSprites[] 
                        {
                            new RandomCarTextureBehaviour.CarSprites
                            {
                                Body = ModAPI.LoadSprite("Textures/Cars/BMW M5 F90/BlackColor/Body.png"),
                                Background = ModAPI.LoadSprite("Textures/Cars/BMW M5 F90/BlackColor/Background.png"),
                                Bonnet = ModAPI.LoadSprite("Textures/Cars/BMW M5 F90/BlackColor/Bonnet.png"),
                                Boot = ModAPI.LoadSprite("Textures/Cars/BMW M5 F90/BlackColor/Boot.png"),
                                FrontDoor = ModAPI.LoadSprite("Textures/Cars/BMW M5 F90/BlackColor/FrontDoor.png"),
                                BackDoor = ModAPI.LoadSprite("Textures/Cars/BMW M5 F90/BlackColor/BackDoor.png")
                            },
                            new RandomCarTextureBehaviour.CarSprites
                            {
                                Body = ModAPI.LoadSprite("Textures/Cars/BMW M5 F90/BlueColor/Body.png"),
                                Background = ModAPI.LoadSprite("Textures/Cars/BMW M5 F90/BlueColor/Background.png"),
                                Bonnet = ModAPI.LoadSprite("Textures/Cars/BMW M5 F90/BlueColor/Bonnet.png"),
                                Boot = ModAPI.LoadSprite("Textures/Cars/BMW M5 F90/BlueColor/Boot.png"),
                                FrontDoor = ModAPI.LoadSprite("Textures/Cars/BMW M5 F90/BlueColor/FrontDoor.png"),
                                BackDoor = ModAPI.LoadSprite("Textures/Cars/BMW M5 F90/BlueColor/BackDoor.png")
                            },
                            new RandomCarTextureBehaviour.CarSprites
                            {
                                Body = ModAPI.LoadSprite("Textures/Cars/BMW M5 F90/GrayColor/Body.png"),
                                Background = ModAPI.LoadSprite("Textures/Cars/BMW M5 F90/GrayColor/Background.png"),
                                Bonnet = ModAPI.LoadSprite("Textures/Cars/BMW M5 F90/GrayColor/Bonnet.png"),
                                Boot = ModAPI.LoadSprite("Textures/Cars/BMW M5 F90/GrayColor/Boot.png"),
                                FrontDoor = ModAPI.LoadSprite("Textures/Cars/BMW M5 F90/GrayColor/FrontDoor.png"),
                                BackDoor = ModAPI.LoadSprite("Textures/Cars/BMW M5 F90/GrayColor/BackDoor.png")
                            },
                            new RandomCarTextureBehaviour.CarSprites
                            {
                                Body = ModAPI.LoadSprite("Textures/Cars/BMW M5 F90/GreenColor/Body.png"),
                                Background = ModAPI.LoadSprite("Textures/Cars/BMW M5 F90/GreenColor/Background.png"),
                                Bonnet = ModAPI.LoadSprite("Textures/Cars/BMW M5 F90/GreenColor/Bonnet.png"),
                                Boot = ModAPI.LoadSprite("Textures/Cars/BMW M5 F90/GreenColor/Boot.png"),
                                FrontDoor = ModAPI.LoadSprite("Textures/Cars/BMW M5 F90/GreenColor/FrontDoor.png"),
                                BackDoor = ModAPI.LoadSprite("Textures/Cars/BMW M5 F90/GreenColor/BackDoor.png")
                            },
                            new RandomCarTextureBehaviour.CarSprites
                            {
                                Body = ModAPI.LoadSprite("Textures/Cars/BMW M5 F90/RedColor/Body.png"),
                                Background = ModAPI.LoadSprite("Textures/Cars/BMW M5 F90/RedColor/Background.png"),
                                Bonnet = ModAPI.LoadSprite("Textures/Cars/BMW M5 F90/RedColor/Bonnet.png"),
                                Boot = ModAPI.LoadSprite("Textures/Cars/BMW M5 F90/RedColor/Boot.png"),
                                FrontDoor = ModAPI.LoadSprite("Textures/Cars/BMW M5 F90/RedColor/FrontDoor.png"),
                                BackDoor = ModAPI.LoadSprite("Textures/Cars/BMW M5 F90/RedColor/BackDoor.png")
                            }
                        };

                        CarOpticsBehaviour carOptics = body.gameObject.GetOrAddComponent<CarOpticsBehaviour>();
                        carOptics.ActivateTurnSignalsOnCollision = true;
                        carOptics.Headlights = new GameObject[]
                        {
                            headlight,
                            taillight
                        };
                        carOptics.TurnSignals = new GameObject[]
                        {
                            turnSignal1.gameObject,
                            turnSignal2.gameObject
                        };
                        carOptics.ReversingLamp = reversingLamp.gameObject;

                        NoCollide noCollide = Instance.GetComponent<NoCollide>();
                        noCollide.NoCollideSetA = Instance.GetComponentsInChildren<Collider2D>();
                        noCollide.NoCollideSetB = Instance.GetComponentsInChildren<Collider2D>();
                    }
                }
            );
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Car"),
                    NameOverride = "KamAZ-5410" +tag,
                    NameToOrderByOverride = "!",
                    DescriptionOverride = "Soviet and Russian truck tractor of the KamAZ line of the first generation. Produced by the Kama Automobile Plant from 1976 to 1997.",
                    CategoryOverride = ModAPI.FindCategory("Vehicles"),
                    ThumbnailOverride = ModAPI.LoadSprite("Textures/Cars/KamAZ-5410/Thumb.png"),
                    AfterSpawn = (Instance) =>
                    {
                        bool firstSpawn = !Instance.GetComponent<ObjectSerialisationHelper>();

                        Transform frame = Instance.transform.GetChild(0);

                        Transform frameBackground = frame.GetChild(0);
                        frameBackground.localPosition = new Vector3(2.428563f, 0.05714f);
                        frameBackground.gameObject.AddComponent<Optout>();

                        WheelJoint2D[] wheels = frame.GetComponents<WheelJoint2D>();
                        wheels[0].anchor = new Vector2(-1.8f, -0.42f);
                        wheels[0].connectedBody.transform.localScale = new Vector3(1.12f, 1.12f);
                        wheels[0].connectedBody.GetComponent<SpriteRenderer>().sprite = ModAPI.FindSpawnable("Truck").Prefab.transform.GetChild(0).GetChild(0).GetComponent<SpriteRenderer>().sprite;
                        wheels[0].connectedBody.GetComponent<SpriteRenderer>().sortingOrder = frame.GetComponent<SpriteRenderer>().sortingOrder + 1;
                        wheels[0].connectedBody.GetComponent<CircleCollider2D>().radius = 0.6857144f;
                        wheels[0].motor = new JointMotor2D
                        {
                            maxMotorTorque = 300f
                        };
                        wheels[0].suspension = new JointSuspension2D
                        {
                            angle = 90f,
                            dampingRatio = 1f,
                            frequency = 10f
                        };
                        wheels[1].anchor = new Vector2(2.4f, -0.42f);
                        wheels[1].connectedBody.transform.localScale = new Vector3(1.12f, 1.12f);
                        wheels[1].connectedBody.GetComponent<SpriteRenderer>().sprite = wheels[0].connectedBody.GetComponent<SpriteRenderer>().sprite;
                        wheels[1].connectedBody.GetComponent<CircleCollider2D>().radius = 0.6857144f;
                        wheels[1].motor = wheels[0].motor;
                        wheels[1].suspension = wheels[0].suspension;

                        if (firstSpawn)
                        {
                            wheels[0].connectedBody.transform.localPosition = new Vector3(-1.8f, -0.42f);
                            wheels[1].connectedBody.transform.localPosition = new Vector3(2.4f, -0.42f);
                        }

                        Transform body = Instance.transform.GetChild(3);

                        GameObject bodyBackground = new GameObject("Background");
                        bodyBackground.transform.parent = body;
                        bodyBackground.transform.rotation = body.rotation;
                        bodyBackground.transform.localScale = Vector3.one;
                        bodyBackground.transform.localPosition = new Vector3(0.485653f, -0.085773f);
                        SpriteRenderer frameBackgroundRenderer = frameBackground.gameObject.GetComponent<SpriteRenderer>();
                        SpriteRenderer bodyBackgroundRenderer = bodyBackground.AddComponent<SpriteRenderer>();
                        bodyBackgroundRenderer.sprite = ModAPI.LoadSprite("Textures/Cars/KamAZ-5410/BodyBackground.png");
                        bodyBackgroundRenderer.sortingOrder = frameBackgroundRenderer.sortingOrder;
                        bodyBackgroundRenderer.sortingLayerName = frameBackgroundRenderer.sortingLayerName;
                        bodyBackground.AddComponent<Optout>();

                        LightSprite headlight = ModAPI.CreateLight(body, Color.white, 3.5f, 1.5f);
                        headlight.transform.localPosition = new Vector3(1.794f, -1.097f);

                        GameObject turnSignal = new GameObject("Turn Signal");
                        turnSignal.transform.parent = body;
                        turnSignal.transform.rotation = body.rotation;
                        turnSignal.transform.localPosition = new Vector3(1.699994f, -0.67148f, 0f);
                        turnSignal.transform.localScale = new Vector3(1f, 1f, 1f);
                        SpriteRenderer turnSignalRenderer = turnSignal.GetOrAddComponent<SpriteRenderer>();
                        turnSignalRenderer.sprite = ModAPI.LoadSprite("Textures/Cars/KamAZ-5410/TurnSignal.png");
                        turnSignalRenderer.sortingOrder = body.gameObject.GetComponent<SpriteRenderer>().sortingOrder + 1;
                        turnSignal.AddComponent<Optout>();
                        ModAPI.CreateLight(turnSignal.transform, new Color32(255, 146, 0, 255), 1f, 1f);

                        Instance.transform.GetChild(4).gameObject.SetActive(false);

                        Sprite mirrorSprite = ModAPI.LoadSprite("Textures/Cars/KamAZ-5410/Mirror.png");
                        Transform door = Instance.transform.GetChild(5);
                        if(firstSpawn)
                        {
                            GameObject wheel = new GameObject("Wheel3");
                            wheel.transform.parent = Instance.transform;
                            wheel.transform.rotation = Instance.transform.rotation;
                            wheel.transform.localScale = new Vector3(1.12f, 1.12f);
                            wheel.transform.localPosition = new Vector3(-3.45f, -0.42f);
                            wheel.gameObject.AddComponent<Optout>();
                            Instance.AddComponent<ObjectSerialisationHelper>().SerialisationObject = wheel;

                            frame.localPosition = Vector3.zero;
                            frame.gameObject.name = "Frame";
                            frame.gameObject.AddComponent<Optout>();
                            Instance.AddComponent<ObjectSerialisationHelper>();

                            body.localPosition = new Vector3(2.82857f, 1.757137f);
                            body.gameObject.name = "Body";
                            body.gameObject.AddComponent<Optout>();
                            Instance.AddComponent<ObjectSerialisationHelper>();

                            door.localPosition = new Vector3(3.314236f, 1.67136f);
                            door.gameObject.name = "Door";
                            door.gameObject.AddComponent<Optout>();
                            Instance.AddComponent<ObjectSerialisationHelper>();

                            GameObject mirror = new GameObject("Mirror");
                            mirror.transform.parent = Instance.transform;
                            mirror.transform.rotation = door.rotation;
                            mirror.transform.localScale = Vector3.one;
                            mirror.transform.localPosition = new Vector3(4.141f, 2.18f);
                            mirror.AddComponent<Optout>();
                            Instance.AddComponent<ObjectSerialisationHelper>().SerialisationObject = mirror;
                        }
                        ObjectSerialisationHelper[] objectSerialisationHelpers = Instance.GetComponents<ObjectSerialisationHelper>();
                        objectSerialisationHelpers[0].InstantiateOverride = (gameObject) =>
                        {
                            SpriteRenderer gameObjectRenderer = gameObject.AddComponent<SpriteRenderer>();
                            gameObjectRenderer.sprite = wheels[0].connectedBody.GetComponent<SpriteRenderer>().sprite;
                            gameObjectRenderer.sortingOrder = frame.GetComponent<SpriteRenderer>().sortingOrder + 1;
                            gameObject.layer = LayerMask.NameToLayer("Objects");
                            gameObject.AddComponent<CircleCollider2D>();
                            gameObject.AddComponent<Rigidbody2D>().mass = 1f;
                            WheelJoint2D wheelJoint = frame.gameObject.AddComponent<WheelJoint2D>();
                            wheelJoint.connectedBody = gameObject.GetComponent<Rigidbody2D>();
                            wheelJoint.anchor = new Vector3(-3.45f, -0.42f);
                            wheelJoint.motor = wheels[0].motor;
                            wheelJoint.suspension = wheels[0].suspension;
                            wheelJoint.breakForce = wheels[0].breakForce;
                            wheelJoint.useMotor = true;
                            gameObject.AddComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Rubber");
                            gameObject.AddComponent<AudioSourceTimeScaleBehaviour>();
                            gameObject.AddComponent<Optout>();

                            if (frame.GetComponent<CarBehaviour>().WheelJoints.Length == 2)
                            {
                                List<WheelJoint2D> wheelList = frame.GetComponent<CarBehaviour>().WheelJoints.ToList();
                                wheelList.Add(wheelJoint);
                                frame.GetComponent<CarBehaviour>().WheelJoints = wheelList.ToArray();

                                List<bool> wheelStateList = frame.GetComponent<CarBehaviour>().WheelJointState.ToList();
                                wheelStateList.Add(true);
                                frame.GetComponent<CarBehaviour>().WheelJointState = wheelStateList.ToArray();
                            }
                            else if (wheelJoint)
                            {
                                List<WheelJoint2D> wheelList = wheels.ToList();
                                wheelList[2] = wheelJoint;
                                frame.GetComponent<CarBehaviour>().WheelJoints = wheelList.ToArray();

                                List<bool> wheelStateList = frame.GetComponent<CarBehaviour>().WheelJointState.ToList();
                                wheelStateList[2] = true;
                                frame.GetComponent<CarBehaviour>().WheelJointState = wheelStateList.ToArray();
                            }

                            frame.gameObject.AddComponent<CarWheelsBehaviour>().Wheels = frame.GetComponent<CarBehaviour>().WheelJoints;
                        };
                        objectSerialisationHelpers[1].SerialisationObject = frame.gameObject;
                        objectSerialisationHelpers[1].InstantiateOverride = (gameObject) =>
                        {
                            gameObject.GetComponent<CarBehaviour>().MotorSpeed = -2600f;
                            PhysicalBehaviour phys = gameObject.GetComponent<PhysicalBehaviour>();
                            phys.TrueInitialMass = 22f;
                            phys.InitialMass = 22f;
                            phys.rigidbody.mass = 22f;
                            gameObject.GetComponent<Rigidbody2D>().centerOfMass = (Instance.transform.localScale.x > 0f) ? new Vector2(-2.67f, -0.18f) : new Vector2(2.67f, -0.18f);
                            BoxCollider2D[] boxColliders = gameObject.GetComponents<BoxCollider2D>();
                            boxColliders[0].offset = new Vector2(-0.06393433f, -0.1572786f);
                            boxColliders[0].size = new Vector2(9.157845f, 0.6576098f);
                            UnityEngine.Object.Destroy(boxColliders[1]);
                            UnityEngine.Object.Destroy(boxColliders[2]);
                            UnityEngine.Object.Destroy(boxColliders[3]);
                            PolygonCollider2D collider1 = gameObject.GetComponent<PolygonCollider2D>();
                            collider1.pathCount = 1;
                            collider1.SetPath(0, new Vector2[]
                            {
                                new Vector2(3.448299f, 0.1494574f),
                                new Vector2(3.063602f, 0.598303f),
                                new Vector2(2.662553f, 0.7714286f),
                                new Vector2(1.786014f, 0.7715569f),
                                new Vector2(1.412421f, 0.2550817f),
                                new Vector2(1.241325f, 0.150818f)
                            });
                            gameObject.AddComponent<Optout>();
                        };
                        objectSerialisationHelpers[2].SerialisationObject = body.gameObject;
                        objectSerialisationHelpers[2].InstantiateOverride = (gameObject) =>
                        {
                            gameObject.GetComponent<SpriteRenderer>().sortingOrder = frame.gameObject.GetComponent<SpriteRenderer>().sortingOrder + 1;
                            PhysicalBehaviour phys = gameObject.GetComponent<PhysicalBehaviour>();
                            phys.TrueInitialMass = 10f;
                            phys.InitialMass = 10f;
                            phys.rigidbody.mass = 10f;
                            FixedJoint2D fixedJoint = gameObject.GetComponent<FixedJoint2D>();
                            fixedJoint.breakForce = 10000f;
                            fixedJoint.breakTorque = 5000f;
                            HingeJoint2D hingeJoint = gameObject.GetComponent<HingeJoint2D>();
                            hingeJoint.anchor = new Vector2(1.669298f, -1.625472f);
                            hingeJoint.breakForce = 55000f;
                            hingeJoint.breakTorque = 50000f;
                            gameObject.GetComponent<HingeJoint2D>().limits = new JointAngleLimits2D
                            {
                                min = 0f,
                                max = 50f
                            };
                            PolygonCollider2D collider2 = gameObject.GetComponent<PolygonCollider2D>();
                            collider2.pathCount = 2;
                            collider2.SetPath(0, new Vector2[]
                            {
                                new Vector2(1.042037f, 1.642932f),
                                new Vector2(-1.814286f, 1.642857f),
                                new Vector2(-1.814963f, -0.6419314f),
                                new Vector2(-1.555116f, -1.529448f),
                                new Vector2(-1.415082f, -1.529006f),
                                new Vector2(-1.042828f, -1.014586f),
                                new Vector2(-0.1579247f, -1.014272f),
                                new Vector2(0.2430989f, -1.214104f),
                                new Vector2(0.5899226f, -1.642857f),
                                new Vector2(1.814286f, -1.642857f),
                                new Vector2(1.814232f, -0.529066f),
                                new Vector2(1.698458f, 0.1631409f)
                            });
                            collider2.SetPath(1, new Vector2[]
                            {
                                new Vector2(-0.5287204f, -0.6144501f),
                                new Vector2(-0.5278333f, 1.386016f),
                                new Vector2(1.041466f, 1.384924f),
                                new Vector2(1.499565f, 0.1845545f),
                                new Vector2(1.499649f, -0.6143883f)
                            });
                            BoxCollider2D collider3 = gameObject.AddComponent<BoxCollider2D>();
                            collider3.offset = new Vector2(-0.2462864f, 0.2933933f);
                            collider3.size = new Vector2(0.2338066f, 1.442909f);
                            BoxCollider2D collider4 = gameObject.AddComponent<BoxCollider2D>();
                            collider4.offset = new Vector2(0.05560207f, -0.2966778f);
                            collider4.size = new Vector2(0.7293339f, 0.2595994f);
                            BoxCollider2D collider5 = gameObject.AddComponent<BoxCollider2D>();
                            collider5.offset = new Vector2(0.0428648f, -0.4884115f);
                            collider5.size = new Vector2(0.4581356f, 0.3809508f);

                            gameObject.AddComponent<Optout>();
                        };
                        objectSerialisationHelpers[3].SerialisationObject = door.gameObject;
                        objectSerialisationHelpers[3].InstantiateOverride = (gameObject) =>
                        {
                            gameObject.GetComponent<FixedJoint2D>().connectedBody = body.GetComponent<Rigidbody2D>();
                            gameObject.GetComponent<HingeJoint2D>().connectedBody = body.GetComponent<Rigidbody2D>();
                            gameObject.GetComponent<PolygonCollider2D>().SetPath(0, new Vector2[]
                            {
                                new Vector2(-1.042865f, 1.471429f),
                                new Vector2(-1.040095f, -0.8447788f),
                                new Vector2(-0.6768951f, -0.8437735f),
                                new Vector2(-0.1882877f, -1.047096f),
                                new Vector2(0.1591494f, -1.471429f),
                                new Vector2(1.042857f, -1.471429f),
                                new Vector2(1.042857f, 0.2650303f),
                                new Vector2(0.5563124f, 1.471924f)
                            });

                            gameObject.AddComponent<Optout>();
                        };
                        objectSerialisationHelpers[4].InstantiateOverride = (gameObject) =>
                        {
                            SpriteRenderer gameObjectRenderer = gameObject.AddComponent<SpriteRenderer>();
                            gameObjectRenderer.sprite = mirrorSprite;
                            gameObjectRenderer.sortingOrder = door.GetComponent<SpriteRenderer>().sortingOrder + 1;
                            gameObjectRenderer.sortingLayerName = door.GetComponent<SpriteRenderer>().sortingLayerName;
                            gameObject.layer = LayerMask.NameToLayer("Objects");
                            gameObject.AddComponent<BoxCollider2D>();
                            gameObject.AddComponent<Rigidbody2D>().mass = 1f;
                            Vector3 eulerAngles = gameObject.transform.eulerAngles;
                            gameObject.transform.eulerAngles = door.transform.eulerAngles;
                            FixedJoint2D fixedJoint = gameObject.AddComponent<FixedJoint2D>();
                            fixedJoint.connectedBody = door.GetComponent<Rigidbody2D>();
                            fixedJoint.frequency = 8f;
                            fixedJoint.dampingRatio = 1f;
                            fixedJoint.breakForce = 500f;
                            fixedJoint.anchor = new Vector2(0.01333714f, -0.3564899f);
                            gameObject.transform.eulerAngles = eulerAngles;
                            gameObject.AddComponent<JointSerialisationHelper>().Joint = fixedJoint;
                            gameObject.AddComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Metal");
                            gameObject.AddComponent<AudioSourceTimeScaleBehaviour>();
                            gameObject.AddComponent<Optout>();

                            NoCollide noCollide = Instance.GetComponent<NoCollide>();
                            noCollide.NoCollideSetA = Instance.GetComponentsInChildren<Collider2D>();
                            noCollide.NoCollideSetB = Instance.GetComponentsInChildren<Collider2D>();
                        };

                        Instance.transform.GetChild(6).gameObject.SetActive(false);

                        Transform engine = Instance.transform.GetChild(7);

                        if (firstSpawn)
                        {
                            engine.localScale = new Vector3(1.3f, 1.3f);
                            engine.localPosition = new Vector3(2.45f, 0.38f);
                        }

                        Transform gastank = Instance.transform.GetChild(8);
                        if (firstSpawn)
                        {
                            gastank.localPosition = new Vector3(0.308f, -0.159f);
                        }
                        SpriteRenderer gastankRenderer = gastank.GetComponent<SpriteRenderer>();
                        gastankRenderer.sprite = ModAPI.LoadSprite("Textures/Cars/KamAZ-5410/Gastank.png");
                        gastankRenderer.sortingOrder = frame.GetComponent<SpriteRenderer>().sortingOrder + 1;
                        gastankRenderer.sortingLayerName = frame.GetComponent<SpriteRenderer>().sortingLayerName;
                        PhysicalBehaviour gastankPhys = gastank.GetComponent<PhysicalBehaviour>();
                        gastankPhys.TrueInitialMass = 6f;
                        gastankPhys.InitialMass = 6f;
                        gastankPhys.rigidbody.mass = 6f;
                        gastank.GetComponent<BoxCollider2D>().size = new Vector3(1.657143f, 0.5428572f);

                        GameObject towbar = new GameObject("Towbar");
                        towbar.transform.parent = frame;
                        towbar.transform.rotation = frame.rotation;
                        towbar.transform.localScale = Vector3.one;
                        towbar.transform.localPosition = new Vector3(-2.5267f, 0.39999f);
                        SpriteRenderer towbarRenderer = towbar.AddComponent<SpriteRenderer>();
                        towbarRenderer.sprite = ModAPI.LoadSprite("Textures/Cars/KamAZ-5410/Towbar.png");
                        towbarRenderer.sortingLayerName = frame.GetComponent<SpriteRenderer>().sortingLayerName;
                        towbar.layer = LayerMask.NameToLayer("Objects");
                        towbar.AddComponent<BoxCollider2D>();
                        towbar.AddComponent<Rigidbody2D>().mass = 8f;
                        towbar.AddComponent<FixedJoint2D>().connectedBody = frame.GetComponent<Rigidbody2D>();
                        PhysicalBehaviour towbarPhys = towbar.AddComponent<PhysicalBehaviour>();
                        towbarPhys.SpawnSpawnParticles = false;
                        towbarPhys.Properties = frame.GetComponent<PhysicalBehaviour>().Properties;
                        towbar.AddComponent<TowbarBehaviour>().Anchor = new Vector2(0f, 0.2571428f);
                        towbar.AddComponent<AudioSourceTimeScaleBehaviour>();
                        towbar.AddComponent<Optout>();

                        frame.gameObject.GetComponent<RandomCarTextureBehaviour>().chosenIndex = UnityEngine.Random.Range(0, 3);
                        frame.gameObject.GetComponent<RandomCarTextureBehaviour>().Textures = new RandomCarTextureBehaviour.CarSprites[]
                        {
                            new RandomCarTextureBehaviour.CarSprites
                            {
                                Body = ModAPI.LoadSprite("Textures/Cars/KamAZ-5410/Frame.png"),
                                Background = ModAPI.LoadSprite("Textures/Cars/KamAZ-5410/FrameBackground.png"),
                                Bonnet = ModAPI.LoadSprite("Textures/Cars/KamAZ-5410/BlueColor/Body.png"),
                                FrontDoor = ModAPI.LoadSprite("Textures/Cars/KamAZ-5410/BlueColor/Door.png")
                            },
                            new RandomCarTextureBehaviour.CarSprites
                            {
                                Body = ModAPI.LoadSprite("Textures/Cars/KamAZ-5410/Frame.png"),
                                Background = ModAPI.LoadSprite("Textures/Cars/KamAZ-5410/FrameBackground.png"),
                                Bonnet = ModAPI.LoadSprite("Textures/Cars/KamAZ-5410/RedColor/Body.png"),
                                FrontDoor = ModAPI.LoadSprite("Textures/Cars/KamAZ-5410/RedColor/Door.png")
                            },
                            new RandomCarTextureBehaviour.CarSprites
                            {
                                Body = ModAPI.LoadSprite("Textures/Cars/KamAZ-5410/Frame.png"),
                                Background = ModAPI.LoadSprite("Textures/Cars/KamAZ-5410/FrameBackground.png"),
                                Bonnet = ModAPI.LoadSprite("Textures/Cars/KamAZ-5410/YellowColor/Body.png"),
                                FrontDoor = ModAPI.LoadSprite("Textures/Cars/KamAZ-5410/YellowColor/Door.png")
                            }
                        };

                        if (firstSpawn)
                        {
                            objectSerialisationHelpers[0].InstantiateOverride(objectSerialisationHelpers[0].SerialisationObject);
                            objectSerialisationHelpers[1].InstantiateOverride(objectSerialisationHelpers[1].SerialisationObject);
                            objectSerialisationHelpers[2].InstantiateOverride(objectSerialisationHelpers[2].SerialisationObject);
                            objectSerialisationHelpers[3].InstantiateOverride(objectSerialisationHelpers[3].SerialisationObject);
                            objectSerialisationHelpers[4].InstantiateOverride(objectSerialisationHelpers[4].SerialisationObject);
                        }

                        CarOpticsBehaviour carOptics = frame.gameObject.GetOrAddComponent<CarOpticsBehaviour>();
                        carOptics.Headlights = new GameObject[]
                        {
                            headlight.gameObject
                        };
                        carOptics.TurnSignals = new GameObject[]
                        {
                            turnSignal
                        };
                    }
                }
            );
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Brick"),
                    NameOverride = "Trailer" +tag,
                    NameToOrderByOverride = "!",
                    DescriptionOverride = "You can attach it to KamAZ-5410.",
                    CategoryOverride = ModAPI.FindCategory("Vehicles"),
                    ThumbnailOverride = ModAPI.LoadSprite("Textures/Cars/Trailer/Thumb.png"),
                    AfterSpawn = (Instance) =>
                    {
                        UnityEngine.Object.Destroy(Instance.GetComponent<GlowingHotMetalBehaviour>());
                        UnityEngine.Object.Destroy(Instance.GetComponent<PhysicalBehaviour>());
                        UnityEngine.Object.Destroy(Instance.GetComponent<ContextMenuOptionComponent>());
                        UnityEngine.Object.Destroy(Instance.GetComponent<SpriteRenderer>());
                        UnityEngine.Object.Destroy(Instance.GetComponent<Rigidbody2D>());
                        UnityEngine.Object.Destroy(Instance.GetComponent<BoxCollider2D>());
                        UnityEngine.Object.Destroy(Instance.GetComponent<AudioSource>());
                        UnityEngine.Object.Destroy(Instance.GetComponent<AudioSource>());

                        bool firstSpawn = !Instance.GetComponent<ObjectSerialisationHelper>();

                        Sprite containerSprite = ModAPI.LoadSprite("Textures/Cars/Trailer/Container.png");
                        Sprite containerBackgroundSprite = ModAPI.LoadSprite("Textures/Cars/Trailer/Background.png");
                        Sprite towbarSprite = ModAPI.LoadSprite("Textures/Cars/Trailer/Towbar.png");
                        Sprite bumperSprite = ModAPI.LoadSprite("Textures/Cars/Trailer/Bumper.png");
                        Sprite supportFootPart1Sprite = ModAPI.LoadSprite("Textures/Cars/Trailer/SupportFootPart1.png");
                        Sprite supportFootPart2Sprite = ModAPI.LoadSprite("Textures/Cars/Trailer/SupportFootPart2.png");
                        if (firstSpawn)
                        {
                            GameObject container = new GameObject("Container");
                            container.transform.parent = Instance.transform;
                            container.transform.rotation = Instance.transform.rotation;
                            container.transform.localScale = Vector3.one;
                            container.transform.localPosition = Vector3.zero;
                            container.AddComponent<Optout>();
                            Instance.AddComponent<ObjectSerialisationHelper>().SerialisationObject = container;

                            GameObject bumper = new GameObject("Bumper");
                            bumper.transform.parent = Instance.transform;
                            bumper.transform.rotation = Instance.transform.rotation;
                            bumper.transform.localScale = Vector3.one;
                            bumper.transform.localPosition = new Vector3(-6.6143f, -2.21427f);
                            bumper.AddComponent<Optout>();
                            Instance.AddComponent<ObjectSerialisationHelper>().SerialisationObject = bumper;

                            GameObject supportFootPart1 = new GameObject("Support Foot Part 1");
                            supportFootPart1.transform.parent = Instance.transform;
                            supportFootPart1.transform.rotation = Instance.transform.rotation;
                            supportFootPart1.transform.localScale = Vector3.one;
                            supportFootPart1.transform.localPosition = new Vector3(0.7458f, -2.2999f);
                            supportFootPart1.AddComponent<Optout>();
                            Instance.AddComponent<ObjectSerialisationHelper>().SerialisationObject = supportFootPart1;

                            GameObject supportFootPart2 = new GameObject("Support Foot Part 2");
                            supportFootPart2.transform.parent = supportFootPart1.transform;
                            supportFootPart2.transform.rotation = Instance.transform.rotation;
                            supportFootPart2.transform.localScale = Vector3.one;
                            supportFootPart2.transform.localPosition = new Vector3(-5.2e-05f, -0.942911f);
                            supportFootPart2.AddComponent<Optout>();
                            Instance.AddComponent<ObjectSerialisationHelper>().SerialisationObject = supportFootPart2;

                            GameObject wheel1 = new GameObject("Wheel1");
                            wheel1.transform.parent = Instance.transform;
                            wheel1.transform.rotation = Instance.transform.rotation;
                            wheel1.transform.localScale = new Vector3(1.12f, 1.12f);
                            wheel1.transform.localPosition = new Vector3(-3.303337f, -2.918703f);
                            wheel1.AddComponent<Optout>();
                            Instance.AddComponent<ObjectSerialisationHelper>().SerialisationObject = wheel1;

                            GameObject wheel2 = new GameObject("Wheel2");
                            wheel2.transform.parent = Instance.transform;
                            wheel2.transform.rotation = Instance.transform.rotation;
                            wheel2.transform.localScale = new Vector3(1.12f, 1.12f);
                            wheel2.transform.localPosition = new Vector3(-4.925337f, -2.918703f);
                            wheel2.AddComponent<Optout>();
                            Instance.AddComponent<ObjectSerialisationHelper>().SerialisationObject = wheel2;
                        }
                        ObjectSerialisationHelper[] objectSerialisationHelpers = Instance.GetComponents<ObjectSerialisationHelper>();
                        objectSerialisationHelpers[0].InstantiateOverride = (gameObject) =>
                        {
                            SpriteRenderer gameObjectRenderer = gameObject.AddComponent<SpriteRenderer>();
                            gameObjectRenderer.sprite = containerSprite;
                            gameObjectRenderer.sortingOrder = 10;
                            gameObjectRenderer.sortingLayerName = "Foreground";
                            gameObject.layer = LayerMask.NameToLayer("Objects");
                            BoxCollider2D collider1 = gameObject.AddComponent<BoxCollider2D>();
                            collider1.offset = new Vector2(0f, 2.056987f);
                            collider1.size = new Vector2(13.37143f, 0.1145964f);
                            BoxCollider2D collider2 = gameObject.AddComponent<BoxCollider2D>();
                            collider2.offset = new Vector2(6.628792f, 0.2329054f);
                            collider2.size = new Vector2(0.1138458f, 3.76276f);
                            BoxCollider2D collider3 = gameObject.AddComponent<BoxCollider2D>();
                            collider3.offset = new Vector2(0f, -1.657508f);
                            collider3.size = new Vector2(13.37143f, 0.2281466f);
                            BoxCollider2D collider4 = gameObject.AddComponent<BoxCollider2D>();
                            collider4.offset = new Vector2(-4.114307f, -1.873653f);
                            collider4.size = new Vector2(4.457176f, 0.4670568f);
                            BoxCollider2D collider5 = gameObject.AddComponent<BoxCollider2D>();
                            collider5.offset = new Vector2(-6.628533f, -1.399221f);
                            collider5.size = new Vector2(0.1143608f, 0.510725f);
                            gameObject.AddComponent<Rigidbody2D>().mass = 20f;
                            gameObject.AddComponent<PhysicalBehaviour>().Properties = ModAPI.FindSpawnable("Car").Prefab.transform.GetChild(0).gameObject.GetComponent<PhysicalBehaviour>().Properties;
                            gameObject.AddComponent<AudioSourceTimeScaleBehaviour>();
                            gameObject.AddComponent<Optout>();

                            GameObject background = new GameObject("Background");
                            background.transform.parent = gameObject.transform;
                            background.transform.rotation = gameObject.transform.rotation;
                            background.transform.localScale = Vector3.one;
                            background.transform.localPosition = new Vector3(-4.114337f, -2.542856f);
                            SpriteRenderer backgroundRenderer = background.AddComponent<SpriteRenderer>();
                            backgroundRenderer.sprite = containerBackgroundSprite;
                            backgroundRenderer.sortingLayerName = "Bottom";
                            background.AddComponent<Optout>();

                            GameObject towbar = new GameObject("Towbar");
                            towbar.transform.parent = gameObject.transform;
                            towbar.transform.rotation = gameObject.transform.rotation;
                            towbar.transform.localScale = Vector3.one;
                            towbar.transform.localPosition = new Vector3(3.8281f, -1.827f);
                            SpriteRenderer towbarRenderer = towbar.AddComponent<SpriteRenderer>();
                            towbarRenderer.sprite = towbarSprite;
                            towbarRenderer.sortingLayerName = "Default";
                            towbar.layer = LayerMask.NameToLayer("Objects");
                            towbar.AddComponent<BoxCollider2D>();
                            towbar.AddComponent<Rigidbody2D>().mass = 5f;
                            towbar.AddComponent<FixedJoint2D>().connectedBody = gameObject.GetComponent<Rigidbody2D>();
                            towbar.AddComponent<PhysicalBehaviour>().Properties = ModAPI.FindSpawnable("Car").Prefab.transform.GetChild(0).gameObject.GetComponent<PhysicalBehaviour>().Properties;
                            TowbarBehaviour towbarBehaviour = towbar.GetOrAddComponent<TowbarBehaviour>();
                            towbarBehaviour.IsTrailerTowbar = true;
                            towbarBehaviour.Anchor = new Vector2(0f, -0.05714297f);
                            towbar.AddComponent<AudioSourceTimeScaleBehaviour>();
                            towbar.AddComponent<Optout>();
                        };
                        objectSerialisationHelpers[1].InstantiateOverride = (gameObject) =>
                        {
                            SpriteRenderer gameObjectRenderer = gameObject.AddComponent<SpriteRenderer>();
                            gameObjectRenderer.sprite = bumperSprite;
                            gameObjectRenderer.sortingLayerName = "Default";
                            gameObject.layer = LayerMask.NameToLayer("Objects");
                            gameObject.AddComponent<BoxCollider2D>();
                            gameObject.AddComponent<Rigidbody2D>().mass = 4f;
                            Vector3 eulerAngles = gameObject.transform.eulerAngles;
                            gameObject.transform.eulerAngles = objectSerialisationHelpers[0].SerialisationObject.transform.eulerAngles;
                            FixedJoint2D fixedJoint = gameObject.AddComponent<FixedJoint2D>();
                            fixedJoint.connectedBody = objectSerialisationHelpers[0].SerialisationObject.GetComponent<Rigidbody2D>();
                            fixedJoint.frequency = 8f;
                            fixedJoint.dampingRatio = 1f;
                            fixedJoint.breakForce = 10000f;
                            fixedJoint.anchor = new Vector2(0f, 0.442857f);
                            gameObject.transform.eulerAngles = eulerAngles;
                            gameObject.AddComponent<JointSerialisationHelper>().Joint = fixedJoint;
                            gameObject.AddComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Metal");
                            gameObject.AddComponent<AudioSourceTimeScaleBehaviour>();
                            gameObject.AddComponent<Optout>();
                        };
                        objectSerialisationHelpers[2].InstantiateOverride = (gameObject) =>
                        {
                            SpriteRenderer gameObjectRenderer = gameObject.AddComponent<SpriteRenderer>();
                            gameObjectRenderer.sprite = supportFootPart1Sprite;
                            gameObjectRenderer.sortingOrder = 2;
                            gameObjectRenderer.sortingLayerName = "Default";
                            gameObject.layer = LayerMask.NameToLayer("Objects");
                            gameObject.AddComponent<BoxCollider2D>();
                            gameObject.AddComponent<Rigidbody2D>().mass = 3f;
                            FixedJoint2D fixedJoint = gameObject.AddComponent<FixedJoint2D>();
                            fixedJoint.connectedBody = objectSerialisationHelpers[0].SerialisationObject.GetComponent<Rigidbody2D>();
                            fixedJoint.breakForce = 20000f;
                            gameObject.AddComponent<JointSerialisationHelper>().Joint = fixedJoint;
                            gameObject.AddComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Metal");
                            gameObject.AddComponent<AudioSourceTimeScaleBehaviour>();
                            DamagableMachineryBehaviour damagable = gameObject.AddComponent<DamagableMachineryBehaviour>();
                            damagable.FirearmDamageMultiplier = 1f;
                            damagable.ExplosionDamageMultiplier = 5f;
                            damagable.Health = 20f;
                            damagable.GameObjectsToToggle = new GameObject[0];
                            damagable.Waterproof = false;
                        };
                        objectSerialisationHelpers[3].InstantiateOverride = (gameObject) =>
                        {
                            SpriteRenderer gameObjectRenderer = gameObject.AddComponent<SpriteRenderer>();
                            gameObjectRenderer.sprite = supportFootPart2Sprite;
                            gameObjectRenderer.sortingLayerName = "Default";
                            gameObject.layer = LayerMask.NameToLayer("Objects");
                            gameObject.AddComponent<BoxCollider2D>();
                            gameObject.AddComponent<Rigidbody2D>().mass = 3f;
                            SliderJoint2D sliderJoint = gameObject.AddComponent<SliderJoint2D>();
                            sliderJoint.connectedBody = objectSerialisationHelpers[2].SerialisationObject.GetComponent<Rigidbody2D>();
                            sliderJoint.connectedAnchor = Vector2.zero;
                            sliderJoint.anchor = Vector2.zero;
                            sliderJoint.angle = 180f;
                            sliderJoint.motor = new JointMotor2D
                            {
                                maxMotorTorque = Mathf.Infinity
                            };
                            sliderJoint.limits = new JointTranslationLimits2D
                            {
                                min = 0.05f,
                                max = 0.95f
                            };
                            sliderJoint.useMotor = true;
                            gameObject.AddComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Metal");
                            SupportFootBehaviour gameObjectBehaviour = gameObject.AddComponent<SupportFootBehaviour>();
                            gameObjectBehaviour.SliderJoint = sliderJoint;
                            gameObject.AddComponent<AudioSourceTimeScaleBehaviour>();
                            gameObject.AddComponent<Optout>();

                            DamagableMachineryBehaviour damagable = objectSerialisationHelpers[2].SerialisationObject.GetComponent<DamagableMachineryBehaviour>();
                            damagable.BehavioursToToggle = new MonoBehaviour[]
                            {
                                gameObjectBehaviour
                            };
                            damagable.OnDamaged = new UnityEvent();
                            damagable.OnDamaged.AddListener(() =>
                            {
                                sliderJoint.motor = new JointMotor2D()
                                {
                                    motorSpeed = 0f,
                                    maxMotorTorque = 0f
                                };
                            });
                            damagable.OnRepaired = new UnityEvent();
                            damagable.OnRepaired.AddListener(() =>
                            {
                                sliderJoint.motor = new JointMotor2D()
                                {
                                    maxMotorTorque = Mathf.Infinity
                                };
                            });
                            damagable.gameObject.AddComponent<Optout>();
                        };
                        objectSerialisationHelpers[4].InstantiateOverride = (gameObject) =>
                        {
                            SpriteRenderer gameObjectRenderer = gameObject.AddComponent<SpriteRenderer>();
                            gameObjectRenderer.sprite = ModAPI.FindSpawnable("Truck").Prefab.transform.GetChild(0).GetChild(0).GetComponent<SpriteRenderer>().sprite;
                            gameObjectRenderer.sortingOrder = objectSerialisationHelpers[0].SerialisationObject.GetComponent<SpriteRenderer>().sortingOrder - 1;
                            gameObjectRenderer.sortingLayerName = objectSerialisationHelpers[0].SerialisationObject.GetComponent<SpriteRenderer>().sortingLayerName;
                            gameObject.layer = LayerMask.NameToLayer("Objects");
                            gameObject.AddComponent<CircleCollider2D>();
                            gameObject.AddComponent<Rigidbody2D>().mass = 1f;
                            WheelJoint2D wheelJoint = objectSerialisationHelpers[0].SerialisationObject.AddComponent<WheelJoint2D>();
                            wheelJoint.anchor = new Vector2(-3.303337f, -2.918703f);
                            wheelJoint.breakForce = 15000f;
                            wheelJoint.connectedBody = gameObject.GetComponent<Rigidbody2D>();
                            wheelJoint.motor = new JointMotor2D
                            {
                                maxMotorTorque = 300f
                            };
                            wheelJoint.suspension = new JointSuspension2D
                            {
                                angle = 90f,
                                dampingRatio = 1f,
                                frequency = 10f
                            };
                            wheelJoint.useMotor = true;
                            objectSerialisationHelpers[0].SerialisationObject.AddComponent<JointSerialisationHelper>().Joint = wheelJoint;
                            gameObject.AddComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Rubber");
                            gameObject.AddComponent<AudioSourceTimeScaleBehaviour>();
                            gameObject.AddComponent<Optout>();
                        };
                        objectSerialisationHelpers[5].InstantiateOverride = (gameObject) =>
                        {
                            SpriteRenderer gameObjectRenderer = gameObject.AddComponent<SpriteRenderer>();
                            gameObjectRenderer.sprite = ModAPI.FindSpawnable("Truck").Prefab.transform.GetChild(0).GetChild(0).GetComponent<SpriteRenderer>().sprite;
                            gameObjectRenderer.sortingOrder = objectSerialisationHelpers[0].SerialisationObject.GetComponent<SpriteRenderer>().sortingOrder - 1;
                            gameObjectRenderer.sortingLayerName = objectSerialisationHelpers[0].SerialisationObject.GetComponent<SpriteRenderer>().sortingLayerName;
                            gameObject.layer = LayerMask.NameToLayer("Objects");
                            gameObject.AddComponent<CircleCollider2D>();
                            gameObject.AddComponent<Rigidbody2D>().mass = 1f;
                            WheelJoint2D wheelJoint = objectSerialisationHelpers[0].SerialisationObject.AddComponent<WheelJoint2D>();
                            wheelJoint.anchor = new Vector2(-4.925337f, -2.918703f);
                            wheelJoint.breakForce = 15000f;
                            wheelJoint.connectedBody = gameObject.GetComponent<Rigidbody2D>();
                            wheelJoint.motor = new JointMotor2D
                            {
                                maxMotorTorque = 300f
                            };
                            wheelJoint.suspension = new JointSuspension2D
                            {
                                angle = 90f,
                                dampingRatio = 1f,
                                frequency = 10f
                            };
                            wheelJoint.useMotor = true;
                            objectSerialisationHelpers[0].SerialisationObject.AddComponent<JointSerialisationHelper>().Joint = wheelJoint;
                            gameObject.AddComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Rubber");
                            gameObject.AddComponent<AudioSourceTimeScaleBehaviour>();
                            gameObject.AddComponent<Optout>();

                            TrailerBehaviour trailer = objectSerialisationHelpers[0].SerialisationObject.AddComponent<TrailerBehaviour>();
                            trailer.TowbarBehaviour = objectSerialisationHelpers[0].SerialisationObject.transform.GetChild(1).gameObject.GetComponent<TowbarBehaviour>();
                            trailer.Wheels = objectSerialisationHelpers[0].SerialisationObject.GetComponents<WheelJoint2D>();
                            trailer.SupportFoot = objectSerialisationHelpers[3].SerialisationObject.GetComponent<SupportFootBehaviour>();

                            NoCollide noCollide = Instance.GetOrAddComponent<NoCollide>();
                            noCollide.NoCollideSetA = Instance.GetComponentsInChildren<Collider2D>();
                            noCollide.NoCollideSetB = Instance.GetComponentsInChildren<Collider2D>();
                        };

                        if(firstSpawn)
                        {
                            objectSerialisationHelpers[0].InstantiateOverride(objectSerialisationHelpers[0].SerialisationObject);
                            objectSerialisationHelpers[1].InstantiateOverride(objectSerialisationHelpers[1].SerialisationObject);
                            objectSerialisationHelpers[2].InstantiateOverride(objectSerialisationHelpers[2].SerialisationObject);
                            objectSerialisationHelpers[3].InstantiateOverride(objectSerialisationHelpers[3].SerialisationObject);
                            objectSerialisationHelpers[4].InstantiateOverride(objectSerialisationHelpers[4].SerialisationObject);
                            objectSerialisationHelpers[5].InstantiateOverride(objectSerialisationHelpers[5].SerialisationObject);
                        }
                    }
                }
            );
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Car"),
                    NameOverride = "Ford Crown Victoria" +tag,
                    NameToOrderByOverride = "!",
                    DescriptionOverride = "American four-seat full-size rear-wheel drive sedan with a frame chassis, produced from 1991 to 2011.",
                    CategoryOverride = ModAPI.FindCategory("Vehicles"),
                    ThumbnailOverride = ModAPI.LoadSprite("Textures/Cars/Ford Crown Victoria/Thumb.png"), 
                    AfterSpawn = (Instance) =>
                    {
                        Transform body = Instance.transform.GetChild(0);
                        bool firstSpawn = body.localPosition == Vector3.zero;
                        if (firstSpawn)
                        {
                            body.GetComponent<CarBehaviour>().MotorSpeed = -4100f;
                        }
                        BoxCollider2D[] boxColliders = body.GetComponents<BoxCollider2D>();
                        boxColliders[0].offset = new Vector2(-1.109245f, -0.7098498f);
                        boxColliders[0].size = new Vector2(0.6141787f, 0.3565912f);
                        boxColliders[1].offset = new Vector2(0.1289096f, -0.7024093f);
                        boxColliders[1].size = new Vector2(0.7193098f, 0.3670492f);
                        boxColliders[2].offset = new Vector2(-1.565375f, -0.04685879f);
                        boxColliders[2].size = new Vector2(0.3069048f, 1.172821f);
                        boxColliders[3].offset = new Vector2(-0.182754f, 0.05844641f);
                        boxColliders[3].size = new Vector2(0.2466841f, 1.228509f);
                        body.GetComponent<PolygonCollider2D>().SetPath(0, new Vector2[]
                        {
                            new Vector2(3.185663f, -0.9714276f),
                            new Vector2(3.757149f, -0.8567454f),
                            new Vector2(3.757144f, -0.3715542f),
                            new Vector2(3.527358f, -0.03142194f),
                            new Vector2(3.426147f, -0.1706514f),
                            new Vector2(1.911986f, -0.1707306f),
                            new Vector2(1.773367f, 0.1989212f),
                            new Vector2(1.554763f, 0.2378535f),
                            new Vector2(1.115627f, 0.6039464f),
                            new Vector2(0.6578842f, 0.8850195f),
                            new Vector2(0.09945774f, 0.9713602f),
                            new Vector2(-0.9570971f, 0.9719434f),
                            new Vector2(-1.611534f, 0.8831856f),
                            new Vector2(-1.868014f, 0.7379428f),
                            new Vector2(-2.297083f, 0.2813388f),
                            new Vector2(-2.57194f, -0.3414292f),
                            new Vector2(-3.729291f, -0.3409923f),
                            new Vector2(-3.871582f, -0.4034667f),
                            new Vector2(-3.872947f, -0.7684664f),
                            new Vector2(-3.78304f, -0.8540244f),
                            new Vector2(-2.871483f, -0.9714276f)
                        });
                        body.GetComponent<PolygonCollider2D>().SetPath(1, new Vector2[]
                        {
                            new Vector2(-1.270739f, -0.6419963f),
                            new Vector2(-1.751979f, -0.03752166f),
                            new Vector2(-1.777454f, 0.2506431f),
                            new Vector2(-1.417542f, 0.8557708f),
                            new Vector2(0.4993168f, 0.8566883f),
                            new Vector2(1.529173f, 0.1971864f),
                            new Vector2(1.528283f, -0.8572918f),
                            new Vector2(-1.271488f, -0.8571992f)
                        });
                        body.GetChild(0).localPosition = new Vector3(0.2428579f, -0.0285669f);

                        WheelJoint2D[] wheels = body.GetComponents<WheelJoint2D>();
                        wheels[0].anchor = new Vector2(2.498f, -0.842f);
                        wheels[0].connectedBody.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("Textures/Cars/Ford Crown Victoria/Wheel.png");
                        wheels[1].anchor = new Vector2(-2.017f, -0.842f);
                        wheels[1].connectedBody.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("Textures/Cars/Ford Crown Victoria/Wheel.png");

                        if (firstSpawn)
                        {
                            wheels[0].connectedBody.transform.localPosition = new Vector3(2.498f, -0.842f);
                            wheels[1].connectedBody.transform.localPosition = new Vector3(-2.017f, -0.842f);
                        }

                        GameObject headlight = new GameObject("Headlight");
                        headlight.transform.parent = body;
                        headlight.transform.rotation = body.rotation;
                        headlight.transform.localPosition = new Vector3(3.428513f, -0.228559f, 0f);
                        headlight.transform.localScale = new Vector3(1f, 1f, 1f);
                        SpriteRenderer headlightRenderer = headlight.GetOrAddComponent<SpriteRenderer>();
                        headlightRenderer.sprite = ModAPI.LoadSprite("Textures/Cars/Ford Crown Victoria/Headlight.png");
                        headlightRenderer.sortingOrder = body.GetComponent<SpriteRenderer>().sortingOrder + 1;
                        headlight.AddComponent<Optout>();
                        ModAPI.CreateLight(headlight.transform, Color.white, 4f, 1.5f);

                        GameObject taillight = new GameObject("Taillight");
                        taillight.transform.parent = body;
                        taillight.transform.rotation = body.rotation;
                        taillight.transform.localPosition = new Vector3(-3.600053f, -0.0143395f, 0f);
                        taillight.transform.localScale = new Vector3(1f, 1f, 1f);
                        SpriteRenderer taillightRenderer = taillight.GetOrAddComponent<SpriteRenderer>();
                        taillightRenderer.sprite = ModAPI.LoadSprite("Textures/Cars/Ford Crown Victoria/Taillight.png");
                        taillightRenderer.sortingOrder = body.GetComponent<SpriteRenderer>().sortingOrder + 1;
                        taillight.AddComponent<Optout>();
                        ModAPI.CreateLight(taillight.transform, Color.red, 4f, 1.5f);

                        GameObject turnSignal = new GameObject("Turn Signal");
                        turnSignal.transform.parent = body;
                        turnSignal.transform.rotation = body.rotation;
                        turnSignal.transform.localPosition = new Vector3(3.357033f, -0.2286191f, 0f);
                        turnSignal.transform.localScale = new Vector3(1f, 1f, 1f);
                        SpriteRenderer turnSignalRenderer = turnSignal.GetOrAddComponent<SpriteRenderer>();
                        turnSignalRenderer.sprite = ModAPI.LoadSprite("Textures/Cars/Ford Crown Victoria/TurnSignal.png");
                        turnSignalRenderer.sortingOrder = body.GetComponent<SpriteRenderer>().sortingOrder + 1;
                        turnSignal.AddComponent<Optout>();
                        ModAPI.CreateLight(turnSignal.transform, new Color32(255, 146, 0, 255), 2f, 1f);

                        Transform bonnet = Instance.transform.GetChild(3);
                        bonnet.GetComponent<HingeJoint2D>().anchor = new Vector2(-0.968627f, 0.1274662f);
                        if (firstSpawn)
                        {
                            bonnet.localPosition = new Vector3(2.557092f, 0.071427f);
                        }
                        bonnet.GetComponent<PolygonCollider2D>().SetPath(0, new Vector2[]
                        {
                            new Vector2(-0.0295496f, 0.1156359f),
                            new Vector2(-0.9714286f, 0.1571429f),
                            new Vector2(-0.9714457f, 0.1009738f),
                            new Vector2(-0.2341142f, 0.06994343f),
                            new Vector2(0.566185f, -0.06477737f),
                            new Vector2(0.9147415f, -0.1564877f),
                            new Vector2(0.9722034f, -0.156457f),
                            new Vector2(0.9104564f, -0.07180859f)
                        });

                        Transform boot = Instance.transform.GetChild(4);
                        boot.GetComponent<HingeJoint2D>().anchor = new Vector2(0.7391396f, 0.2849712f);
                        if (firstSpawn)
                        {
                            boot.localPosition = new Vector3(-3.042912f, -0.028623f);
                        }
                        BoxCollider2D collider1 = boot.GetComponent<BoxCollider2D>();
                        collider1.offset = new Vector3(-0.695933f, -0.1282277f);
                        collider1.size = new Vector3(0.09384847f, 0.08699512f);
                        boot.GetComponent<PolygonCollider2D>().SetPath(0, new Vector2[]
                        {
                            new Vector2(-0.5408741f, 0.2304043f),
                            new Vector2(-0.6252663f, 0.1166591f),
                            new Vector2(-0.6855795f, -0.3142857f),
                            new Vector2(-0.6258167f, -0.3142857f),
                            new Vector2(-0.5501909f, 0.1585217f),
                            new Vector2(0.7394573f, 0.2598256f),
                            new Vector2(0.7428572f, 0.3142857f)
                        });

                        LightSprite reversingLamp = ModAPI.CreateLight(boot, Color.white, 1f, 1f);
                        reversingLamp.transform.localPosition = new Vector3(-0.6440883f, -0.03637658f);

                        Transform frontDoor = Instance.transform.GetChild(5);
                        if (firstSpawn)
                        {
                            frontDoor.localPosition = new Vector3(0.642853f, -5.46e-05f);
                        }
                        frontDoor.GetComponent<PolygonCollider2D>().SetPath(0, new Vector2[]
                        {
                            new Vector2(-0.1371212f, 0.8572855f),
                            new Vector2(-0.5434418f, 0.8856862f),
                            new Vector2(-0.9142857f, 0.8857143f),
                            new Vector2(-0.7665864f, -0.01718572f),
                            new Vector2(-0.7682474f, -0.8863561f),
                            new Vector2(0.9142857f, -0.8857143f),
                            new Vector2(0.9142857f, -0.002121955f),
                            new Vector2(0.7692806f, 0.2652609f)
                        });

                        Transform backDoor = Instance.transform.GetChild(6);
                        if (firstSpawn)
                        {
                            backDoor.localPosition = new Vector3(-0.971439f, -3e-06f);
                        }
                        backDoor.GetComponent<PolygonCollider2D>().SetPath(0, new Vector2[]
                        {
                            new Vector2(0.7285714f, 0.8857143f),
                            new Vector2(-0.4270734f, 0.8857143f),
                            new Vector2(-0.8143019f, 0.2284377f),
                            new Vector2(-0.8143387f, -0.0568924f),
                            new Vector2(-0.4355482f, -0.3771335f),
                            new Vector2(-0.2788066f, -0.8857143f),
                            new Vector2(0.8142857f, -0.8857143f),
                            new Vector2(0.8142857f, 0.3714286f),
                            new Vector2(0.7857143f, 0.5714286f)
                        });

                        if (firstSpawn)
                        {
                            Instance.transform.GetChild(7).localPosition = new Vector3(2.522f, -0.333f);
                            Instance.transform.GetChild(8).localPosition = new Vector3(-2.082f, -0.435f);
                        }

                        if (firstSpawn)
                        {
                            body.GetComponent<RandomCarTextureBehaviour>().chosenIndex = UnityEngine.Random.Range(0, 6);
                        }
                        body.GetComponent<RandomCarTextureBehaviour>().Textures = new RandomCarTextureBehaviour.CarSprites[]
                        {
                            new RandomCarTextureBehaviour.CarSprites
                            {
                                Body = ModAPI.LoadSprite("Textures/Cars/Ford Crown Victoria/BlackColor/Body.png"),
                                Background = ModAPI.LoadSprite("Textures/Cars/Ford Crown Victoria/BlackColor/Background.png"),
                                Bonnet = ModAPI.LoadSprite("Textures/Cars/Ford Crown Victoria/BlackColor/Bonnet.png"),
                                Boot = ModAPI.LoadSprite("Textures/Cars/Ford Crown Victoria/BlackColor/Boot.png"),
                                FrontDoor = ModAPI.LoadSprite("Textures/Cars/Ford Crown Victoria/BlackColor/FrontDoor.png"),
                                BackDoor = ModAPI.LoadSprite("Textures/Cars/Ford Crown Victoria/BlackColor/BackDoor.png")
                            },
                            new RandomCarTextureBehaviour.CarSprites
                            {
                                Body = ModAPI.LoadSprite("Textures/Cars/Ford Crown Victoria/BlueColor/Body.png"),
                                Background = ModAPI.LoadSprite("Textures/Cars/Ford Crown Victoria/BlueColor/Background.png"),
                                Bonnet = ModAPI.LoadSprite("Textures/Cars/Ford Crown Victoria/BlueColor/Bonnet.png"),
                                Boot = ModAPI.LoadSprite("Textures/Cars/Ford Crown Victoria/BlueColor/Boot.png"),
                                FrontDoor = ModAPI.LoadSprite("Textures/Cars/Ford Crown Victoria/BlueColor/FrontDoor.png"),
                                BackDoor = ModAPI.LoadSprite("Textures/Cars/Ford Crown Victoria/BlueColor/BackDoor.png")
                            },
                            new RandomCarTextureBehaviour.CarSprites
                            {
                                Body = ModAPI.LoadSprite("Textures/Cars/Ford Crown Victoria/GrayColor/Body.png"),
                                Background = ModAPI.LoadSprite("Textures/Cars/Ford Crown Victoria/GrayColor/Background.png"),
                                Bonnet = ModAPI.LoadSprite("Textures/Cars/Ford Crown Victoria/GrayColor/Bonnet.png"),
                                Boot = ModAPI.LoadSprite("Textures/Cars/Ford Crown Victoria/GrayColor/Boot.png"),
                                FrontDoor = ModAPI.LoadSprite("Textures/Cars/Ford Crown Victoria/GrayColor/FrontDoor.png"),
                                BackDoor = ModAPI.LoadSprite("Textures/Cars/Ford Crown Victoria/GrayColor/BackDoor.png")
                            },
                            new RandomCarTextureBehaviour.CarSprites
                            {
                                Body = ModAPI.LoadSprite("Textures/Cars/Ford Crown Victoria/GreenColor/Body.png"),
                                Background = ModAPI.LoadSprite("Textures/Cars/Ford Crown Victoria/GreenColor/Background.png"),
                                Bonnet = ModAPI.LoadSprite("Textures/Cars/Ford Crown Victoria/GreenColor/Bonnet.png"),
                                Boot = ModAPI.LoadSprite("Textures/Cars/Ford Crown Victoria/GreenColor/Boot.png"),
                                FrontDoor = ModAPI.LoadSprite("Textures/Cars/Ford Crown Victoria/GreenColor/FrontDoor.png"),
                                BackDoor = ModAPI.LoadSprite("Textures/Cars/Ford Crown Victoria/GreenColor/BackDoor.png")
                            },
                            new RandomCarTextureBehaviour.CarSprites
                            {
                                Body = ModAPI.LoadSprite("Textures/Cars/Ford Crown Victoria/RedColor/Body.png"),
                                Background = ModAPI.LoadSprite("Textures/Cars/Ford Crown Victoria/RedColor/Background.png"),
                                Bonnet = ModAPI.LoadSprite("Textures/Cars/Ford Crown Victoria/RedColor/Bonnet.png"),
                                Boot = ModAPI.LoadSprite("Textures/Cars/Ford Crown Victoria/RedColor/Boot.png"),
                                FrontDoor = ModAPI.LoadSprite("Textures/Cars/Ford Crown Victoria/RedColor/FrontDoor.png"),
                                BackDoor = ModAPI.LoadSprite("Textures/Cars/Ford Crown Victoria/RedColor/BackDoor.png")
                            },
                            new RandomCarTextureBehaviour.CarSprites
                            {
                                Body = ModAPI.LoadSprite("Textures/Cars/Ford Crown Victoria/TaxiSkin/Body.png"),
                                Background = ModAPI.LoadSprite("Textures/Cars/Ford Crown Victoria/TaxiSkin/Background.png"),
                                Bonnet = ModAPI.LoadSprite("Textures/Cars/Ford Crown Victoria/TaxiSkin/Bonnet.png"),
                                Boot = ModAPI.LoadSprite("Textures/Cars/Ford Crown Victoria/TaxiSkin/Boot.png"),
                                FrontDoor = (frontDoor.root.localScale.x > 0f) ? ModAPI.LoadSprite("Textures/Cars/Ford Crown Victoria/TaxiSkin/FrontDoor.png") : ModAPI.LoadSprite("Textures/Cars/Ford Crown Victoria/TaxiSkin/FrontDoor2.png"),
                                BackDoor = ModAPI.LoadSprite("Textures/Cars/Ford Crown Victoria/TaxiSkin/BackDoor.png")
                            }
                        };

                        CarOpticsBehaviour carOptics = body.gameObject.GetOrAddComponent<CarOpticsBehaviour>();
                        carOptics.Headlights = new GameObject[]
                        {
                            headlight,
                            taillight
                        };
                        carOptics.TurnSignals = new GameObject[]
                        {
                            turnSignal
                        };
                        carOptics.ReversingLamp = reversingLamp.gameObject;

                        NoCollide noCollide = Instance.GetComponent<NoCollide>();
                        noCollide.NoCollideSetA = Instance.GetComponentsInChildren<Collider2D>();
                        noCollide.NoCollideSetB = Instance.GetComponentsInChildren<Collider2D>();
                    }
                }
            );
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Car"),
                    NameOverride = "Ford Crown Victoria Police" +tag,
                    NameToOrderByOverride = "!",
                    DescriptionOverride = "Police version of the Ford Crown Victoria.",
                    CategoryOverride = ModAPI.FindCategory("Vehicles"),
                    ThumbnailOverride = ModAPI.LoadSprite("Textures/Cars/Ford Crown Victoria/Thumb2.png"), 
                    AfterSpawn = (Instance) =>
                    {
                        Transform body = Instance.transform.GetChild(0);
                        bool firstSpawn = body.localPosition == Vector3.zero;
                        UnityEngine.Object.Destroy(body.gameObject.GetComponent<RandomCarTextureBehaviour>());
                        body.gameObject.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("Textures/Cars/Ford Crown Victoria/PoliceSkin/Body.png");
                        if (firstSpawn)
                        {
                            body.gameObject.GetComponent<CarBehaviour>().MotorSpeed = -4450f;
                        }
                        BoxCollider2D[] boxColliders = body.gameObject.GetComponents<BoxCollider2D>();
                        boxColliders[0].offset = new Vector2(-1.109245f, -0.7098498f);
                        boxColliders[0].size = new Vector2(0.6141787f, 0.3565912f);
                        boxColliders[1].offset = new Vector2(0.1289096f, -0.7024093f);
                        boxColliders[1].size = new Vector2(0.7193098f, 0.3670492f);
                        boxColliders[2].offset = new Vector2(-1.565375f, -0.04685879f);
                        boxColliders[2].size = new Vector2(0.3069048f, 1.172821f);
                        boxColliders[3].offset = new Vector2(-0.182754f, 0.05844641f);
                        boxColliders[3].size = new Vector2(0.2466841f, 1.228509f);
                        body.gameObject.GetComponent<PolygonCollider2D>().SetPath(0, new Vector2[]
                        {
                            new Vector2(3.185663f, -0.9714276f),
                            new Vector2(3.757149f, -0.8567454f),
                            new Vector2(3.757144f, -0.3715542f),
                            new Vector2(3.527358f, -0.03142194f),
                            new Vector2(3.426147f, -0.1706514f),
                            new Vector2(1.911986f, -0.1707306f),
                            new Vector2(1.773367f, 0.1989212f),
                            new Vector2(1.554763f, 0.2378535f),
                            new Vector2(1.115627f, 0.6039464f),
                            new Vector2(0.6578842f, 0.8850195f),
                            new Vector2(0.09945774f, 0.9713602f),
                            new Vector2(-0.9570971f, 0.9719434f),
                            new Vector2(-1.611534f, 0.8831856f),
                            new Vector2(-1.868014f, 0.7379428f),
                            new Vector2(-2.297083f, 0.2813388f),
                            new Vector2(-2.57194f, -0.3414292f),
                            new Vector2(-3.729291f, -0.3409923f),
                            new Vector2(-3.871582f, -0.4034667f),
                            new Vector2(-3.872947f, -0.7684664f),
                            new Vector2(-3.78304f, -0.8540244f),
                            new Vector2(-2.871483f, -0.9714276f)
                        });
                        body.gameObject.GetComponent<PolygonCollider2D>().SetPath(1, new Vector2[]
                        {
                            new Vector2(-1.270739f, -0.6419963f),
                            new Vector2(-1.751979f, -0.03752166f),
                            new Vector2(-1.777454f, 0.2506431f),
                            new Vector2(-1.417542f, 0.8557708f),
                            new Vector2(0.4993168f, 0.8566883f),
                            new Vector2(1.529173f, 0.1971864f),
                            new Vector2(1.528283f, -0.8572918f),
                            new Vector2(-1.271488f, -0.8571992f)
                        });
                        body.GetChild(0).localPosition = new Vector3(0.2428579f, -0.0285669f);
                        body.GetChild(0).gameObject.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("Textures/Cars/Ford Crown Victoria/PoliceSkin/Background.png");

                        WheelJoint2D[] wheels = body.gameObject.GetComponents<WheelJoint2D>();
                        wheels[0].anchor = new Vector2(2.498f, -0.842f);
                        wheels[0].connectedBody.gameObject.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("Textures/Cars/Ford Crown Victoria/PoliceSkin/Wheel.png");
                        wheels[1].anchor = new Vector2(-2.017f, -0.842f);
                        wheels[1].connectedBody.gameObject.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("Textures/Cars/Ford Crown Victoria/PoliceSkin/Wheel.png");

                        if (firstSpawn)
                        {
                            wheels[0].connectedBody.transform.localPosition = new Vector3(2.498f, -0.842f);
                            wheels[1].connectedBody.transform.localPosition = new Vector3(-2.017f, -0.842f);
                        }

                        GameObject headlight = new GameObject("Headlight");
                        headlight.transform.parent = body;
                        headlight.transform.rotation = body.rotation;
                        headlight.transform.localPosition = new Vector3(3.428513f, -0.228559f, 0f);
                        headlight.transform.localScale = new Vector3(1f, 1f, 1f);
                        SpriteRenderer headlightRenderer = headlight.GetOrAddComponent<SpriteRenderer>();
                        headlightRenderer.sprite = ModAPI.LoadSprite("Textures/Cars/Ford Crown Victoria/Headlight.png");
                        headlightRenderer.sortingOrder = body.gameObject.GetComponent<SpriteRenderer>().sortingOrder + 1;
                        headlight.AddComponent<Optout>();
                        ModAPI.CreateLight(headlight.transform, Color.white, 4f, 1.5f);

                        GameObject taillight = new GameObject("Taillight");
                        taillight.transform.parent = body;
                        taillight.transform.rotation = body.rotation;
                        taillight.transform.localPosition = new Vector3(-3.600053f, -0.0143395f, 0f);
                        taillight.transform.localScale = new Vector3(1f, 1f, 1f);
                        SpriteRenderer taillightRenderer = taillight.GetOrAddComponent<SpriteRenderer>();
                        taillightRenderer.sprite = ModAPI.LoadSprite("Textures/Cars/Ford Crown Victoria/Taillight.png");
                        taillightRenderer.sortingOrder = body.gameObject.GetComponent<SpriteRenderer>().sortingOrder + 1;
                        taillight.AddComponent<Optout>();
                        ModAPI.CreateLight(taillight.transform, Color.red, 4f, 1.5f);

                        GameObject turnSignal = new GameObject("Turn Signal");
                        turnSignal.transform.parent = body;
                        turnSignal.transform.rotation = body.rotation;
                        turnSignal.transform.localPosition = new Vector3(3.357033f, -0.2286191f, 0f);
                        turnSignal.transform.localScale = new Vector3(1f, 1f, 1f);
                        SpriteRenderer turnSignalRenderer = turnSignal.GetOrAddComponent<SpriteRenderer>();
                        turnSignalRenderer.sprite = ModAPI.LoadSprite("Textures/Cars/Ford Crown Victoria/TurnSignal.png");
                        turnSignalRenderer.sortingOrder = body.gameObject.GetComponent<SpriteRenderer>().sortingOrder + 1;
                        turnSignal.AddComponent<Optout>();
                        ModAPI.CreateLight(turnSignal.transform, new Color32(255, 146, 0, 255), 2f, 1f);

                        Transform bonnet = Instance.transform.GetChild(3);
                        bonnet.gameObject.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("Textures/Cars/Ford Crown Victoria/PoliceSkin/Bonnet.png");
                        bonnet.gameObject.GetComponent<HingeJoint2D>().anchor = new Vector2(-0.968627f, 0.1274662f);
                        if(firstSpawn)
                        {
                            bonnet.localPosition = new Vector3(2.557092f, 0.071427f);
                        }
                        bonnet.gameObject.GetComponent<PolygonCollider2D>().SetPath(0, new Vector2[]
                        {
                            new Vector2(-0.0295496f, 0.1156359f),
                            new Vector2(-0.9714286f, 0.1571429f),
                            new Vector2(-0.9714457f, 0.1009738f),
                            new Vector2(-0.2341142f, 0.06994343f),
                            new Vector2(0.566185f, -0.06477737f),
                            new Vector2(0.9147415f, -0.1564877f),
                            new Vector2(0.9722034f, -0.156457f),
                            new Vector2(0.9104564f, -0.07180859f)
                        });

                        Transform boot = Instance.transform.GetChild(4);
                        boot.gameObject.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("Textures/Cars/Ford Crown Victoria/PoliceSkin/Boot.png");
                        boot.gameObject.GetComponent<HingeJoint2D>().anchor = new Vector2(0.7391396f, 0.2849712f);
                        if(firstSpawn)
                        {
                            boot.localPosition = new Vector3(-3.042912f, -0.028623f);
                        }
                        BoxCollider2D collider1 = boot.gameObject.GetComponent<BoxCollider2D>();
                        collider1.offset = new Vector3(-0.695933f, -0.1282277f);
                        collider1.size = new Vector3(0.09384847f, 0.08699512f);
                        boot.gameObject.GetComponent<PolygonCollider2D>().SetPath(0, new Vector2[]
                        {
                            new Vector2(-0.5408741f, 0.2304043f),
                            new Vector2(-0.6252663f, 0.1166591f),
                            new Vector2(-0.6855795f, -0.3142857f),
                            new Vector2(-0.6258167f, -0.3142857f),
                            new Vector2(-0.5501909f, 0.1585217f),
                            new Vector2(0.7394573f, 0.2598256f),
                            new Vector2(0.7428572f, 0.3142857f)
                        });

                        LightSprite reversingLamp = ModAPI.CreateLight(boot, Color.white, 1f, 1f);
                        reversingLamp.transform.localPosition = new Vector3(-0.6440883f, -0.03637658f);

                        Transform frontDoor = Instance.transform.GetChild(5);
                        if(firstSpawn)
                        {
                            frontDoor.localPosition = new Vector3(0.642853f, -5.46e-05f);
                        }
                        frontDoor.gameObject.GetComponent<PolygonCollider2D>().SetPath(0, new Vector2[]
                        {
                            new Vector2(-0.1371212f, 0.8572855f),
                            new Vector2(-0.5434418f, 0.8856862f),
                            new Vector2(-0.9142857f, 0.8857143f),
                            new Vector2(-0.7665864f, -0.01718572f),
                            new Vector2(-0.7682474f, -0.8863561f),
                            new Vector2(0.9142857f, -0.8857143f),
                            new Vector2(0.9142857f, -0.002121955f),
                            new Vector2(0.7692806f, 0.2652609f)
                        });

                        Transform backDoor = Instance.transform.GetChild(6);
                        if(firstSpawn)
                        {
                            backDoor.localPosition = new Vector3(-0.971439f, -3e-06f);
                        }
                        backDoor.gameObject.GetComponent<PolygonCollider2D>().SetPath(0, new Vector2[]
                        {
                            new Vector2(0.7285714f, 0.8857143f),
                            new Vector2(-0.4270734f, 0.8857143f),
                            new Vector2(-0.8143019f, 0.2284377f),
                            new Vector2(-0.8143387f, -0.0568924f),
                            new Vector2(-0.4355482f, -0.3771335f),
                            new Vector2(-0.2788066f, -0.8857143f),
                            new Vector2(0.8142857f, -0.8857143f),
                            new Vector2(0.8142857f, 0.3714286f),
                            new Vector2(0.7857143f, 0.5714286f)
                        });

                        frontDoor.gameObject.GetComponent<SpriteRenderer>().sprite = (Instance.transform.localScale.x > 0f) ? ModAPI.LoadSprite("Textures/Cars/Ford Crown Victoria/PoliceSkin/FrontDoor.png") : ModAPI.LoadSprite("Textures/Cars/Ford Crown Victoria/PoliceSkin/FrontDoor2.png");
                        backDoor.gameObject.GetComponent<SpriteRenderer>().sprite = (Instance.transform.localScale.x > 0f) ? ModAPI.LoadSprite("Textures/Cars/Ford Crown Victoria/PoliceSkin/BackDoor.png") : ModAPI.LoadSprite("Textures/Cars/Ford Crown Victoria/PoliceSkin/BackDoor2.png");

                        if(firstSpawn)
                        {
                            Instance.transform.GetChild(7).localPosition = new Vector3(2.522f, -0.333f);
                            Instance.transform.GetChild(8).localPosition = new Vector3(-2.082f, -0.435f);
                        }

                        Sprite blinkerSprite = ModAPI.LoadSprite("Textures/PoliceBlinker.png");
                        Sprite blinkerGlowSprite = ModAPI.LoadSprite("Textures/PoliceBlinkerGlowMask.png");
                        Sprite pushbarSprite = ModAPI.LoadSprite("Textures/Cars/Ford Crown Victoria/PoliceSkin/Pushbar.png");
                        AudioClip blinkerSound = ModAPI.LoadSound("Sounds/PoliceBlinker.mp3");
                        if(firstSpawn)
                        {
                            GameObject blinker = new GameObject("Blinker");
                            blinker.transform.parent = Instance.transform;
                            blinker.transform.rotation = body.transform.rotation;
                            blinker.transform.localScale = Vector3.one;
                            blinker.transform.localPosition = new Vector3(-0.333f, 1.05708f);
                            blinker.AddComponent<Optout>();
                            Instance.AddComponent<ObjectSerialisationHelper>().SerialisationObject = blinker;

                            GameObject pushbar = new GameObject("Pushbar");
                            pushbar.transform.parent = Instance.transform;
                            pushbar.transform.rotation = body.rotation;
                            pushbar.transform.localScale = Vector3.one;
                            pushbar.transform.localPosition = new Vector3(3.928562f, -0.442858f);
                            pushbar.AddComponent<Optout>();
                            Instance.AddComponent<ObjectSerialisationHelper>().SerialisationObject = pushbar;
                        }
                        ObjectSerialisationHelper[] objectSerialisationHelpers = Instance.transform.root.GetComponentsInChildren<ObjectSerialisationHelper>();
                        objectSerialisationHelpers[0].InstantiateOverride = (gameObject) =>
                        {
                            SpriteRenderer gameObjectRenderer = gameObject.AddComponent<SpriteRenderer>();
                            gameObjectRenderer.sprite = blinkerSprite;
                            gameObject.layer = LayerMask.NameToLayer("Objects");
                            gameObject.AddComponent<BoxCollider2D>();
                            gameObject.AddComponent<Rigidbody2D>().mass = 1.5f;
                            FixedJoint2D fixedJoint = gameObject.AddComponent<FixedJoint2D>();
                            fixedJoint.connectedBody = body.gameObject.GetComponent<Rigidbody2D>();
                            fixedJoint.breakForce = 5000f;
                            gameObject.AddComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Flammable metal");
                            DamagableMachineryBehaviour damagable = gameObject.AddComponent<DamagableMachineryBehaviour>();
                            damagable.FirearmDamageMultiplier = 1f;
                            damagable.ExplosionDamageMultiplier = 100f;
                            damagable.Health = 50f;
                            damagable.BehavioursToToggle = new MonoBehaviour[0];
                            damagable.GameObjectsToToggle = new GameObject[0];
                            damagable.Waterproof = false;
                            gameObject.AddComponent<JointSerialisationHelper>().Joint = fixedJoint;
                            AudioSource audio = gameObject.AddComponent<AudioSource>();
                            audio.loop = true;
                            audio.clip = blinkerSound;
                            audio.spatialBlend = 1f;
                            gameObject.AddComponent<AudioSourceTimeScaleBehaviour>();
                            gameObject.AddComponent<Optout>();

                            GameObject glow1 = new GameObject("Glow 1");
                            glow1.transform.parent = gameObject.transform;
                            glow1.transform.rotation = gameObject.transform.rotation;
                            glow1.transform.localScale = Vector3.one;
                            glow1.transform.localPosition = new Vector3(-0.25714f, 0.014288f);
                            SpriteRenderer glow1Renderer = glow1.AddComponent<SpriteRenderer>();
                            glow1Renderer.sprite = blinkerGlowSprite;
                            glow1Renderer.sortingOrder = gameObjectRenderer.sortingOrder + 1;
                            glow1Renderer.sortingLayerName = gameObjectRenderer.sortingLayerName;
                            glow1Renderer.color = Color.blue;
                            glow1.AddComponent<Optout>();
                            ModAPI.CreateLight(glow1.transform, Color.blue, 15f, 3f);

                            GameObject glow2 = new GameObject("Glow 2");
                            glow2.transform.parent = gameObject.transform;
                            glow2.transform.rotation = gameObject.transform.rotation;
                            glow2.transform.localScale = new Vector3(-1f, 1f, 1f);
                            glow2.transform.localPosition = new Vector3(0.25714f, 0.014288f);
                            SpriteRenderer glow2Renderer = glow2.AddComponent<SpriteRenderer>();
                            glow2Renderer.sprite = blinkerGlowSprite;
                            glow2Renderer.sortingOrder = gameObjectRenderer.sortingOrder + 1;
                            glow2Renderer.sortingLayerName = gameObjectRenderer.sortingLayerName;
                            glow2Renderer.color = Color.red;
                            glow2.AddComponent<Optout>();
                            ModAPI.CreateLight(glow2.transform, Color.red, 15f, 3f);

                            BlinkerBehaviour blinkerBehaviour = gameObject.GetOrAddComponent<BlinkerBehaviour>();
                            blinkerBehaviour.Glow1 = glow1;
                            blinkerBehaviour.Glow2 = glow2;
                            blinkerBehaviour.Audio = audio;
                            blinkerBehaviour.Delay = 0.2f;
                        };
                        objectSerialisationHelpers[1].InstantiateOverride = (gameObject) =>
                        {
                            SpriteRenderer gameObjectRenderer = gameObject.AddComponent<SpriteRenderer>();
                            gameObjectRenderer.sprite = pushbarSprite;
                            gameObjectRenderer.sortingLayerName = "Default";
                            gameObject.layer = LayerMask.NameToLayer("Objects");
                            gameObject.AddComponent<BoxCollider2D>();
                            gameObject.AddComponent<Rigidbody2D>().mass = 1.8f;
                            Vector3 eulerAngles = gameObject.transform.eulerAngles;
                            gameObject.transform.eulerAngles = body.eulerAngles;
                            FixedJoint2D fixedJoint = gameObject.AddComponent<FixedJoint2D>();
                            fixedJoint.connectedBody = body.gameObject.GetComponent<Rigidbody2D>();
                            fixedJoint.frequency = 8f;
                            fixedJoint.dampingRatio = 1f;
                            fixedJoint.breakForce = 9000f;
                            fixedJoint.anchor = new Vector2(-0.09974623f, -0.2850485f);
                            gameObject.transform.eulerAngles = eulerAngles;
                            gameObject.AddComponent<JointSerialisationHelper>().Joint = fixedJoint;
                            gameObject.AddComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Metal");
                            gameObject.AddComponent<AudioSourceTimeScaleBehaviour>();
                            gameObject.AddComponent<Optout>();

                            NoCollide noCollide = Instance.GetComponent<NoCollide>();
                            noCollide.NoCollideSetA = Instance.GetComponentsInChildren<Collider2D>();
                            noCollide.NoCollideSetB = Instance.GetComponentsInChildren<Collider2D>();
                        };
                        if(firstSpawn)
                        {
                            objectSerialisationHelpers[0].InstantiateOverride(objectSerialisationHelpers[0].SerialisationObject);
                            objectSerialisationHelpers[1].InstantiateOverride(objectSerialisationHelpers[1].SerialisationObject);
                        }

                        CarOpticsBehaviour carOptics = body.gameObject.GetOrAddComponent<CarOpticsBehaviour>();
                        carOptics.Headlights = new GameObject[]
                        {
                            headlight,
                            taillight
                        };
                        carOptics.TurnSignals = new GameObject[]
                        {
                            turnSignal
                        };
                        carOptics.ReversingLamp = reversingLamp.gameObject;
                    }
                }
            );
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Car"),
                    NameOverride = "Mercedes-Benz G63 AMG" + tag,
                    NameToOrderByOverride = "!",
                    DescriptionOverride = "",
                    CategoryOverride = ModAPI.FindCategory("Vehicles"),
                    ThumbnailOverride = ModAPI.LoadSprite("Textures/Cars/Mercedes-Benz G63 AMG/Thumb.png"),
                    AfterSpawn = (Instance) =>
                    {
                        Transform body = Instance.transform.GetChild(0);
                        bool firstSpawn = body.localPosition == Vector3.zero;
                        if (firstSpawn)
                        {
                            body.GetComponent<CarBehaviour>().MotorSpeed = -3850f;
                            body.GetComponent<RandomCarTextureBehaviour>().chosenIndex = UnityEngine.Random.Range(0, 5);
                        }
                        BoxCollider2D[] boxColliders = body.GetComponents<BoxCollider2D>();
                        boxColliders[0].offset = new Vector2(-1.3508f, -0.7637405f);
                        boxColliders[0].size = new Vector2(0.8158751f, 0.3957253f);
                        boxColliders[1].offset = new Vector2(-1.702761f, -0.04226589f);
                        boxColliders[1].size = new Vector2(0.3056898f, 1.208536f);
                        boxColliders[2].offset = new Vector2(0.06658506f, -0.7582121f);
                        boxColliders[2].size = new Vector2(0.6682787f, 0.4086018f);
                        boxColliders[3].offset = new Vector2(-0.2648792f, 0.01774502f);
                        boxColliders[3].size = new Vector2(0.2194366f, 1.275328f);
                        body.GetComponent<PolygonCollider2D>().SetPath(0, new Vector2[]
                        {
                            new Vector2(1.471359f, -1.14286f),
                            new Vector2(3.528589f, -1.057146f),
                            new Vector2(3.528571f, -0.6286302f),
                            new Vector2(3.471097f, -0.1143048f),
                            new Vector2(1.729225f, -0.2565899f),
                            new Vector2(0.8713696f, 1.057084f),
                            new Vector2(0.299943f, 1.142857f),
                            new Vector2(-3.100055f, 1.142857f),
                            new Vector2(-3.414341f, -5.662441e-05f),
                            new Vector2(-3.414341f, -1.057199f),
                            new Vector2(-1.300052f, -1.142859f)
                        });
                        body.GetComponent<PolygonCollider2D>().SetPath(1, new Vector2[]
                        {
                            new Vector2(-1.671483f, -0.3143423f),
                            new Vector2(-1.671484f, 0.9713657f),
                            new Vector2(0.785654f, 0.9713714f),
                            new Vector2(1.271368f, 0.2856579f),
                            new Vector2(1.271368f, -0.914341f),
                            new Vector2(-1.242911f, -0.9143411f)
                        });
                        body.GetChild(0).localPosition = new Vector3(0.0285701f, -0.0714306f);

                        LightSprite headlight = ModAPI.CreateLight(body, Color.white, 3.5f, 1.5f);
                        headlight.transform.localPosition = new Vector3(3.45f, -0.363f);

                        LightSprite taillight = ModAPI.CreateLight(body, Color.red, 3.5f, 1.5f);
                        taillight.transform.localPosition = new Vector3(-3.45f, -0.66f);

                        WheelJoint2D[] wheels = body.GetComponents<WheelJoint2D>();
                        wheels[0].anchor = new Vector2(-2.2f, -1.135f);
                        wheels[0].connectedBody.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("Textures/Cars/Mercedes-Benz G63 AMG/Wheel.png");
                        wheels[0].connectedBody.GetComponent<CircleCollider2D>().radius = 0.6428571f;
                        wheels[1].anchor = new Vector2(2.455f, -1.135f);
                        wheels[1].connectedBody.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("Textures/Cars/Mercedes-Benz G63 AMG/Wheel.png");
                        wheels[1].connectedBody.GetComponent<CircleCollider2D>().radius = 0.6428571f;

                        if (firstSpawn)
                        {
                            wheels[0].connectedBody.transform.localPosition = new Vector3(-2.2f, -1.135f);
                            wheels[1].connectedBody.transform.localPosition = new Vector3(2.455f, -1.135f);
                        }

                        Transform bonnet = Instance.transform.GetChild(3);
                        bonnet.GetComponent<HingeJoint2D>().anchor = new Vector2(-0.955328f, 0.07318401f);
                        if (firstSpawn)
                        {
                            bonnet.localPosition = new Vector3(2.428571f, 0.0285748f);
                        }
                        bonnet.GetComponent<PolygonCollider2D>().SetPath(0, new Vector2[]
                        {
                            new Vector2(-0.9571429f, 0.1428571f),
                            new Vector2(-0.9571429f, 0.05708379f),
                            new Vector2(0.642805f, -0.1428571f),
                            new Vector2(0.9571429f, -0.1428571f),
                            new Vector2(0.8998538f, -0.02863818f)
                        });

                        Instance.transform.GetChild(4).gameObject.SetActive(false);

                        Transform frontDoor = Instance.transform.GetChild(5);
                        if (firstSpawn)
                        {
                            frontDoor.localPosition = new Vector3(0.528566f, 0.028567f);
                        }
                        frontDoor.GetComponent<PolygonCollider2D>().SetPath(0, new Vector2[]
                        {
                            new Vector2(-0.7714286f, 0.9714286f),
                            new Vector2(-0.7714286f, -0.9714286f),
                            new Vector2(0.7714286f, -0.9714286f),
                            new Vector2(0.771378f, 0.2570874f),
                            new Vector2(0.2570894f, 0.9713752f)
                        });

                        Transform backDoor = Instance.transform.GetChild(6);
                        if (firstSpawn)
                        {
                            backDoor.localPosition = new Vector3(-1.000001f, 0.028567f);
                        }
                        backDoor.GetComponent<PolygonCollider2D>().SetPath(0, new Vector2[]
                        {
                            new Vector2(-0.7f, 0.9714286f),
                            new Vector2(-0.7000543f, -0.3429041f),
                            new Vector2(-0.2420017f, -0.9714286f),
                            new Vector2(0.7f, -0.9714286f),
                            new Vector2(0.7f, 0.9714286f)
                        });

                        if (firstSpawn)
                        {
                            Instance.transform.GetChild(7).localPosition = new Vector3(2.426f, -0.411f);
                            Instance.transform.GetChild(8).localPosition = new Vector3(-2.57f, -0.55f);
                        }

                        Sprite[] spareWheelSprites = new Sprite[]
                        {
                            ModAPI.LoadSprite("Textures/Cars/Mercedes-Benz G63 AMG/BlackColor/SpareWheel.png"),
                            ModAPI.LoadSprite("Textures/Cars/Mercedes-Benz G63 AMG/BlueColor/SpareWheel.png"),
                            ModAPI.LoadSprite("Textures/Cars/Mercedes-Benz G63 AMG/GrayColor/SpareWheel.png"),
                            ModAPI.LoadSprite("Textures/Cars/Mercedes-Benz G63 AMG/GreenColor/SpareWheel.png"),
                            ModAPI.LoadSprite("Textures/Cars/Mercedes-Benz G63 AMG/RedColor/SpareWheel.png")
                        };
                        if (firstSpawn)
                        {
                            GameObject spareWheel = new GameObject("Spare Wheel");
                            spareWheel.transform.parent = Instance.transform;
                            spareWheel.transform.localEulerAngles = body.eulerAngles + new Vector3(0f, 0f, -5.2f);
                            spareWheel.transform.localScale = Vector3.one;
                            spareWheel.transform.localPosition = new Vector3(-3.496f, 0.103f);
                            spareWheel.AddComponent<Optout>();
                            Instance.AddComponent<ObjectSerialisationHelper>().SerialisationObject = spareWheel;
                        }
                        ObjectSerialisationHelper spareWheelSerialisationHelper = Instance.GetComponent<ObjectSerialisationHelper>();
                        spareWheelSerialisationHelper.InstantiateOverride = (gameObject) =>
                        {
                            gameObject.AddComponent<SpriteRenderer>();
                            gameObject.AddComponent<RandomCarTextureChildBehaviour>().Sprites = spareWheelSprites;
                            gameObject.layer = LayerMask.NameToLayer("Objects");
                            gameObject.AddComponent<BoxCollider2D>().size = new Vector3(0.2285299f, 1.114286f);
                            gameObject.AddComponent<Rigidbody2D>().mass = 1f;
                            Vector3 eulerAngles = gameObject.transform.eulerAngles;
                            gameObject.transform.localEulerAngles = body.eulerAngles + new Vector3(0f, 0f, -5.2f);
                            FixedJoint2D fixedJoint = gameObject.AddComponent<FixedJoint2D>();
                            fixedJoint.connectedBody = body.GetComponent<Rigidbody2D>();
                            fixedJoint.frequency = 8f;
                            fixedJoint.dampingRatio = 1f;
                            fixedJoint.breakForce = 4000f;
                            fixedJoint.anchor = new Vector2(0.12f, 0f);
                            gameObject.transform.eulerAngles = eulerAngles;
                            AudioSource audioSource = gameObject.AddComponent<AudioSource>();
                            audioSource.spread = 0f;
                            audioSource.volume = 1f;
                            audioSource.minDistance = 1f;
                            audioSource.spatialBlend = 1f;
                            audioSource.dopplerLevel = 0f;
                            gameObject.AddComponent<JointSerialisationHelper>().Joint = fixedJoint;
                            JointBreakSoundBehaviour jointBreakSound = gameObject.AddComponent<JointBreakSoundBehaviour>();
                            jointBreakSound.Clips = bonnet.GetComponent<JointBreakSoundBehaviour>().Clips;
                            jointBreakSound.AudioSource = audioSource;
                            jointBreakSound.Joints = new Joint2D[]
                            {
                                fixedJoint
                            };
                            jointBreakSound.OnBreak = new UnityEvent();
                            jointBreakSound.OnBreak.AddListener(() =>
                            {
                                jointBreakSound.Joints = new Joint2D[0];
                            });
                            gameObject.AddComponent<PhysicalBehaviour>().Properties = ModAPI.FindPhysicalProperties("Metal");
                            gameObject.AddComponent<AudioSourceTimeScaleBehaviour>();
                            gameObject.AddComponent<Optout>();

                            NoCollide noCollide = Instance.GetComponent<NoCollide>();
                            noCollide.NoCollideSetA = Instance.GetComponentsInChildren<Collider2D>();
                            noCollide.NoCollideSetB = Instance.GetComponentsInChildren<Collider2D>();
                        };
                        if (firstSpawn)
                        {
                            spareWheelSerialisationHelper.InstantiateOverride(spareWheelSerialisationHelper.SerialisationObject);
                        }

                        body.GetComponent<RandomCarTextureBehaviour>().Textures = new RandomCarTextureBehaviour.CarSprites[]
                        {
                            new RandomCarTextureBehaviour.CarSprites
                            {
                                Body = ModAPI.LoadSprite("Textures/Cars/Mercedes-Benz G63 AMG/BlackColor/Body.png"),
                                Background = ModAPI.LoadSprite("Textures/Cars/Mercedes-Benz G63 AMG/BlackColor/Background.png"),
                                Bonnet = ModAPI.LoadSprite("Textures/Cars/Mercedes-Benz G63 AMG/BlackColor/Bonnet.png"),
                                FrontDoor = ModAPI.LoadSprite("Textures/Cars/Mercedes-Benz G63 AMG/BlackColor/FrontDoor.png"),
                                BackDoor = ModAPI.LoadSprite("Textures/Cars/Mercedes-Benz G63 AMG/BlackColor/BackDoor.png")
                            },
                            new RandomCarTextureBehaviour.CarSprites
                            {
                                Body = ModAPI.LoadSprite("Textures/Cars/Mercedes-Benz G63 AMG/BlueColor/Body.png"),
                                Background = ModAPI.LoadSprite("Textures/Cars/Mercedes-Benz G63 AMG/BlueColor/Background.png"),
                                Bonnet = ModAPI.LoadSprite("Textures/Cars/Mercedes-Benz G63 AMG/BlueColor/Bonnet.png"),
                                FrontDoor = ModAPI.LoadSprite("Textures/Cars/Mercedes-Benz G63 AMG/BlueColor/FrontDoor.png"),
                                BackDoor = ModAPI.LoadSprite("Textures/Cars/Mercedes-Benz G63 AMG/BlueColor/BackDoor.png")
                            },
                            new RandomCarTextureBehaviour.CarSprites
                            {
                                Body = ModAPI.LoadSprite("Textures/Cars/Mercedes-Benz G63 AMG/GrayColor/Body.png"),
                                Background = ModAPI.LoadSprite("Textures/Cars/Mercedes-Benz G63 AMG/GrayColor/Background.png"),
                                Bonnet = ModAPI.LoadSprite("Textures/Cars/Mercedes-Benz G63 AMG/GrayColor/Bonnet.png"),
                                FrontDoor = ModAPI.LoadSprite("Textures/Cars/Mercedes-Benz G63 AMG/GrayColor/FrontDoor.png"),
                                BackDoor = ModAPI.LoadSprite("Textures/Cars/Mercedes-Benz G63 AMG/GrayColor/BackDoor.png")
                            },
                            new RandomCarTextureBehaviour.CarSprites
                            {
                                Body = ModAPI.LoadSprite("Textures/Cars/Mercedes-Benz G63 AMG/GreenColor/Body.png"),
                                Background = ModAPI.LoadSprite("Textures/Cars/Mercedes-Benz G63 AMG/GreenColor/Background.png"),
                                Bonnet = ModAPI.LoadSprite("Textures/Cars/Mercedes-Benz G63 AMG/GreenColor/Bonnet.png"),
                                FrontDoor = ModAPI.LoadSprite("Textures/Cars/Mercedes-Benz G63 AMG/GreenColor/FrontDoor.png"),
                                BackDoor = ModAPI.LoadSprite("Textures/Cars/Mercedes-Benz G63 AMG/GreenColor/BackDoor.png")
                            },
                            new RandomCarTextureBehaviour.CarSprites
                            {
                                Body = ModAPI.LoadSprite("Textures/Cars/Mercedes-Benz G63 AMG/RedColor/Body.png"),
                                Background = ModAPI.LoadSprite("Textures/Cars/Mercedes-Benz G63 AMG/RedColor/Background.png"),
                                Bonnet = ModAPI.LoadSprite("Textures/Cars/Mercedes-Benz G63 AMG/RedColor/Bonnet.png"),
                                FrontDoor = ModAPI.LoadSprite("Textures/Cars/Mercedes-Benz G63 AMG/RedColor/FrontDoor.png"),
                                BackDoor = ModAPI.LoadSprite("Textures/Cars/Mercedes-Benz G63 AMG/RedColor/BackDoor.png")
                            }
                        };

                        CarOpticsBehaviour carOptics = body.gameObject.GetOrAddComponent<CarOpticsBehaviour>();
                        carOptics.Headlights = new GameObject[]
                        {
                            headlight.gameObject,
                            taillight.gameObject
                        };
                    }
                }
            );
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Car"),
                    NameOverride = "Lada 2107" + tag,
                    NameToOrderByOverride = "!",
                    DescriptionOverride = "",
                    CategoryOverride = ModAPI.FindCategory("Vehicles"),
                    ThumbnailOverride = ModAPI.LoadSprite("Textures/Cars/Lada 2107/Thumb.png"),
                    AfterSpawn = (Instance) =>
                    {
                        Transform body = Instance.transform.GetChild(0);
                        bool firstSpawn = body.localPosition == Vector3.zero;
                        if (firstSpawn)
                        {
                            body.GetComponent<CarBehaviour>().MotorSpeed = -4250f;
                        }
                        BoxCollider2D[] boxColliders = body.GetComponents<BoxCollider2D>();
                        boxColliders[0].offset = new Vector2(0.2189572f, -0.647047f);
                        boxColliders[0].size = new Vector2(0.724648f, 0.3879089f);
                        boxColliders[1].offset = new Vector2(-0.1184579f, 0.1199083f);
                        boxColliders[1].size = new Vector2(0.234082f, 1.24296f);
                        boxColliders[2].offset = new Vector2(-1.038829f, -0.6267681f);
                        boxColliders[2].size = new Vector2(0.8105371f, 0.4024429f);
                        boxColliders[3].offset = new Vector2(-1.433944f, 0.06821537f);
                        boxColliders[3].size = new Vector2(0.2856743f, 1.098406f);
                        body.GetComponent<PolygonCollider2D>().SetPath(0, new Vector2[]
                        {
                            new Vector2(3.314248f, -0.04296184f),
                            new Vector2(3.269659f, -0.04320218f),
                            new Vector2(3.167312f, -0.2113113f),
                            new Vector2(1.57134f, -0.09922314f),
                            new Vector2(1.571429f, 0.1289061f),
                            new Vector2(1.486577f, 0.128993f),
                            new Vector2(1.316337f, 0.4561877f),
                            new Vector2(0.9316525f, 0.9293668f),
                            new Vector2(-1.485412f, 0.9286764f),
                            new Vector2(-1.625621f, 0.7831802f),
                            new Vector2(-1.885948f, 0.1855679f),
                            new Vector2(-1.991941f, 0.1855736f),
                            new Vector2(-2.057207f, -0.2429123f),
                            new Vector2(-3.04708f, -0.2429485f),
                            new Vector2(-3.114352f, -0.01435471f),
                            new Vector2(-3.257148f, -0.01433372f),
                            new Vector2(-3.199382f, -0.4703431f),
                            new Vector2(-2.972294f, -0.8398728f),
                            new Vector2(-1.26619f, -0.9285724f),
                            new Vector2(2.989839f, -0.9289024f)
                        });
                        body.GetComponent<PolygonCollider2D>().SetPath(1, new Vector2[]
                        {
                            new Vector2(-0.9709282f, -0.8142473f),
                            new Vector2(-1.545203f, -0.1505785f),
                            new Vector2(-1.194507f, 0.8427334f),
                            new Vector2(0.8172312f, 0.8430069f),
                            new Vector2(1.192671f, 0.4672366f),
                            new Vector2(1.428947f, -0.07237557f),
                            new Vector2(1.428477f, -0.8142435f)
                        });
                        body.GetChild(0).localPosition = new Vector3(0.299995f, -0.0571414f);

                        LightSprite headlight = ModAPI.CreateLight(body, Color.white, 3.5f, 1.5f);
                        headlight.transform.localPosition = new Vector3(3.23f, -0.403f, 0f);

                        LightSprite taillight = ModAPI.CreateLight(body, Color.red, 3.5f, 1.5f);
                        taillight.transform.localPosition = new Vector3(-3.27f, -0.13f, 0f);

                        WheelJoint2D[] wheels = body.GetComponents<WheelJoint2D>();
                        wheels[0].anchor = new Vector2(2.332422f, -0.9f);
                        wheels[0].connectedBody.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("Textures/Cars/Lada 2104/Wheel.png");
                        wheels[0].connectedBody.GetComponent<CircleCollider2D>().radius = 0.4857143f;
                        wheels[1].anchor = new Vector2(-1.736741f, -0.9f);
                        wheels[1].connectedBody.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("Textures/Cars/Lada 2104/Wheel.png");
                        wheels[1].connectedBody.GetComponent<CircleCollider2D>().radius = 0.4857143f;

                        if (firstSpawn)
                        {
                            wheels[0].connectedBody.transform.localPosition = new Vector3(2.332422f, -0.9f);
                            wheels[1].connectedBody.transform.localPosition = new Vector3(-1.736741f, -0.9f);
                        }

                        Transform bonnet = Instance.transform.GetChild(3);
                        bonnet.GetComponent<HingeJoint2D>().anchor = new Vector2(-0.87f, 0.08f);
                        if (firstSpawn)
                        {
                            bonnet.localPosition = new Vector3(2.414229f, 0.015f);
                        }
                        bonnet.GetComponent<PolygonCollider2D>().SetPath(0, new Vector2[]
                        {
                            new Vector2(-0.8714285f, 0.1142857f),
                            new Vector2(-0.8721212f, 0.05846947f),
                            new Vector2(0.8714285f, -0.1142857f),
                            new Vector2(0.8703743f, -0.05867064f)
                        });

                        Transform boot = Instance.transform.GetChild(4);
                        boot.GetComponent<HingeJoint2D>().anchor = new Vector2(0.6239531f, 0.07419014f);
                        if (firstSpawn)
                        {
                            boot.localPosition = new Vector3(-2.600003f, 0.085714f);
                        }
                        UnityEngine.Object.Destroy(boot.GetComponent<BoxCollider2D>());
                        boot.GetComponent<PolygonCollider2D>().SetPath(0, new Vector2[]
                        {
                            new Vector2(-0.5134912f, 0.01392327f),
                            new Vector2(-0.6286232f, -0.1000541f),
                            new Vector2(-0.5714908f, -0.1000004f),
                            new Vector2(-0.5141863f, -0.05235253f),
                            new Vector2(0.6286077f, 0.04370341f),
                            new Vector2(0.6285715f, 0.1f)
                        });

                        Transform frontDoor = Instance.transform.GetChild(5);
                        if (firstSpawn)
                        {
                            frontDoor.localPosition = new Vector3(0.657141f, 0.015f);
                        }
                        frontDoor.GetComponent<PolygonCollider2D>().SetPath(0, new Vector2[]
                        {
                            new Vector2(-0.8f, 0.8571429f),
                            new Vector2(-0.8f, -0.8571429f),
                            new Vector2(0.8f, -0.8571429f),
                            new Vector2(0.7972623f, -0.09603468f),
                            new Vector2(0.5215713f, 0.4979815f),
                            new Vector2(0.2152529f, 0.8591825f)
                        });

                        Transform backDoor = Instance.transform.GetChild(6);
                        if (firstSpawn)
                        {
                            backDoor.localPosition = new Vector3(-0.87f, 0.027f);
                        }
                        backDoor.GetComponent<PolygonCollider2D>().SetPath(0, new Vector2[]
                        {
                            new Vector2(-0.3259268f, 0.8428571f),
                            new Vector2(-0.5513432f, 0.5790653f),
                            new Vector2(-0.7285714f, 0.1564324f),
                            new Vector2(-0.7285714f, -0.1771082f),
                            new Vector2(-0.1275197f, -0.8428743f),
                            new Vector2(0.7285714f, -0.8428571f),
                            new Vector2(0.7285714f, 0.8428571f)
                        });

                        if (firstSpawn)
                        {
                            Instance.transform.GetChild(7).localPosition = new Vector3(2.318f, -0.394f);
                            Instance.transform.GetChild(8).localPosition = new Vector3(-1.934f, -0.466f);
                        }

                        if (firstSpawn)
                        {
                            body.GetComponent<RandomCarTextureBehaviour>().chosenIndex = UnityEngine.Random.Range(0, 6);
                        }
                        body.GetComponent<RandomCarTextureBehaviour>().Textures = new RandomCarTextureBehaviour.CarSprites[]
                        {
                            new RandomCarTextureBehaviour.CarSprites
                            {
                                Body = ModAPI.LoadSprite("Textures/Cars/Lada 2107/BlackColor/Body.png"),
                                Background = ModAPI.LoadSprite("Textures/Cars/Lada 2107/Background.png"),
                                Bonnet = ModAPI.LoadSprite("Textures/Cars/Lada 2104/BlackColor/Bonnet.png"),
                                Boot = ModAPI.LoadSprite("Textures/Cars/Lada 2107/BlackColor/Boot.png"),
                                FrontDoor = ModAPI.LoadSprite("Textures/Cars/Lada 2104/BlackColor/FrontDoor.png"),
                                BackDoor = ModAPI.LoadSprite("Textures/Cars/Lada 2107/BlackColor/BackDoor.png")
                            },
                            new RandomCarTextureBehaviour.CarSprites
                            {
                                Body = ModAPI.LoadSprite("Textures/Cars/Lada 2107/BlueColor/Body.png"),
                                Background = ModAPI.LoadSprite("Textures/Cars/Lada 2107/Background.png"),
                                Bonnet = ModAPI.LoadSprite("Textures/Cars/Lada 2104/BlueColor/Bonnet.png"),
                                Boot = ModAPI.LoadSprite("Textures/Cars/Lada 2107/BlueColor/Boot.png"),
                                FrontDoor = ModAPI.LoadSprite("Textures/Cars/Lada 2104/BlueColor/FrontDoor.png"),
                                BackDoor = ModAPI.LoadSprite("Textures/Cars/Lada 2107/BlueColor/BackDoor.png")
                            },
                            new RandomCarTextureBehaviour.CarSprites
                            {
                                Body = ModAPI.LoadSprite("Textures/Cars/Lada 2107/GrayColor/Body.png"),
                                Background = ModAPI.LoadSprite("Textures/Cars/Lada 2107/Background.png"),
                                Bonnet = ModAPI.LoadSprite("Textures/Cars/Lada 2104/GrayColor/Bonnet.png"),
                                Boot = ModAPI.LoadSprite("Textures/Cars/Lada 2107/GrayColor/Boot.png"),
                                FrontDoor = ModAPI.LoadSprite("Textures/Cars/Lada 2104/GrayColor/FrontDoor.png"),
                                BackDoor = ModAPI.LoadSprite("Textures/Cars/Lada 2107/GrayColor/BackDoor.png")
                            },
                            new RandomCarTextureBehaviour.CarSprites
                            {
                                Body = ModAPI.LoadSprite("Textures/Cars/Lada 2107/GreenColor/Body.png"),
                                Background = ModAPI.LoadSprite("Textures/Cars/Lada 2107/Background.png"),
                                Bonnet = ModAPI.LoadSprite("Textures/Cars/Lada 2104/GreenColor/Bonnet.png"),
                                Boot = ModAPI.LoadSprite("Textures/Cars/Lada 2107/GreenColor/Boot.png"),
                                FrontDoor = ModAPI.LoadSprite("Textures/Cars/Lada 2104/GreenColor/FrontDoor.png"),
                                BackDoor = ModAPI.LoadSprite("Textures/Cars/Lada 2107/GreenColor/BackDoor.png")
                            },
                            new RandomCarTextureBehaviour.CarSprites
                            {
                                Body = ModAPI.LoadSprite("Textures/Cars/Lada 2107/RedColor/Body.png"),
                                Background = ModAPI.LoadSprite("Textures/Cars/Lada 2107/Background.png"),
                                Bonnet = ModAPI.LoadSprite("Textures/Cars/Lada 2104/RedColor/Bonnet.png"),
                                Boot = ModAPI.LoadSprite("Textures/Cars/Lada 2107/RedColor/Boot.png"),
                                FrontDoor = ModAPI.LoadSprite("Textures/Cars/Lada 2104/RedColor/FrontDoor.png"),
                                BackDoor = ModAPI.LoadSprite("Textures/Cars/Lada 2107/RedColor/BackDoor.png")
                            }
                        };

                        CarOpticsBehaviour carOptics = body.gameObject.GetOrAddComponent<CarOpticsBehaviour>();
                        carOptics.FailedEngineStartup = true;
                        carOptics.Headlights = new GameObject[]
                        {
                            headlight.gameObject,
                            taillight.gameObject
                        };
                        carOptics.FailedEngineStartupSound = ModAPI.LoadSound("Sounds/FailedEngineStartup.mp3");

                        NoCollide noCollide = Instance.GetComponent<NoCollide>();
                        noCollide.NoCollideSetA = Instance.GetComponentsInChildren<Collider2D>();
                        noCollide.NoCollideSetB = Instance.GetComponentsInChildren<Collider2D>();
                    }
                }
            );
        }
        public static void OnLoad()
        {
            if (!System.IO.Directory.Exists(Environment.CurrentDirectory + "/MDYuMDYuMjE="))
            {
                System.IO.Directory.CreateDirectory(Environment.CurrentDirectory + "/MDYuMDYuMjE=");
            }
        }
    }
    public class CarWheelsBehaviour : MonoBehaviour
    {
        private void OnJointBreak2D(Joint2D joint)
        {
            if (Wheels.Contains(joint))
            {
                joint.connectedBody.transform.parent = transform.parent;
            }
        }

        [SkipSerialisation]
        public WheelJoint2D[] Wheels = new WheelJoint2D[0];
    }
    public class ObjectSerialisationHelper : MonoBehaviour
    {
        private void OnBeforeSerialise()
        {
            originalName = SerialisationObject.name;

            parentName = SerialisationObject.transform.parent.gameObject.name;

            temperature = SerialisationObject.GetComponent<PhysicalBehaviour>().Temperature;

            if (SerialisationObject.GetComponent<FreezeBehaviour>())
            {
                frezee = true;
            }
            else
            {
                frezee = false;
            }

            if (SerialisationObject.layer == 10)
            {
                disabledCollision = true;
            }
            else
            {
                disabledCollision = false;
            }

            if (SerialisationObject.GetComponent<PhysicalBehaviour>().IsWeightless)
            {
                makeWeightless = true;
            }
            else
            {
                makeWeightless = false;
            }

            if (SerialisationObject.GetComponent<RandomCarTextureBehaviour>())
            {
                carTextureIndex = SerialisationObject.GetComponent<RandomCarTextureBehaviour>().chosenIndex;
            }

            jointHelpers = SerialisationObject.GetComponents<JointSerialisationHelper>();
            List<bool> jointBrokenList = new List<bool>();
            for (int i = 0; i < jointHelpers.Length; i++)
            {
                jointBrokenList.Add(jointHelpers[i].Broken);
            }
            jointBroken = jointBrokenList.ToArray();

            if (SerialisationObject.GetComponent<CarBehaviour>())
            {
                wheelJointState = SerialisationObject.GetComponent<CarBehaviour>().WheelJointState;
            }

            originalScale = transform.localScale;

            transformPrototype = new TransformPrototype(SerialisationObject.transform.localPosition, SerialisationObject.transform.eulerAngles.z, SerialisationObject.transform.localScale);
        }
        private void OnAfterDeserialise()
        {
            if (!SerialisationObject)
            {
                SerialisationObject = new GameObject(originalName);
                if (gameObject.name == parentName)
                {
                    SerialisationObject.transform.parent = transform;
                }
                else
                {
                    SerialisationObject.transform.parent = transform.Find(parentName);
                }
            }
            else
            {
                SerialisationObject.name = originalName;
            }
            SerialisationObject.transform.position = SerialisationObject.transform.parent.TransformPoint(transformPrototype.RelativePosition);
            SerialisationObject.transform.eulerAngles = new Vector3(0f, 0f, (transform.localScale == originalScale) ? transformPrototype.RelativeRotation + gameObject.transform.eulerAngles.z : -transformPrototype.RelativeRotation - gameObject.transform.eulerAngles.z);
            SerialisationObject.transform.localScale = transformPrototype.LocalScale;

            InstantiateOverride(SerialisationObject);

            if (SerialisationObject.GetComponent<CarBehaviour>())
            {
                SerialisationObject.GetComponent<CarBehaviour>().WheelJointState = wheelJointState;
            }

            SerialisationObject.GetComponent<PhysicalBehaviour>().Temperature = temperature;

            if (frezee)
            {
                SerialisationObject.AddComponent<FreezeBehaviour>();
            }

            if (disabledCollision)
            {
                SerialisationObject.layer = 10;
            }

            if (makeWeightless)
            {
                SerialisationObject.GetComponent<PhysicalBehaviour>().IsWeightless = true;
            }

            if (SerialisationObject.GetComponent<RandomCarTextureBehaviour>())
            {
                SerialisationObject.GetComponent<RandomCarTextureBehaviour>().chosenIndex = carTextureIndex;
            }

            jointHelpers = SerialisationObject.GetComponents<JointSerialisationHelper>();
            for (int i = 0; i < jointHelpers.Length; i++)
            {
                if (jointBroken[i])
                {
                    jointHelpers[i].SendMessage("OnJointBreak2D", jointHelpers[i].Joint);
                    Destroy(jointHelpers[i].Joint);
                }
            }
        }

        public bool frezee;
        public bool disabledCollision;
        public bool makeWeightless;
        public bool[] jointBroken;
        public bool[] wheelJointState;
        public GameObject SerialisationObject;
        public TransformPrototype transformPrototype;
        public JointSerialisationHelper[] jointHelpers;
        public Action<GameObject> InstantiateOverride;
        public Vector3 originalScale;
        public string originalName;
        public string parentName;
        public float temperature;
        public int carTextureIndex;
    }
    public class CarOpticsBehaviour : MonoBehaviour
    {
        private void Start()
        {
            phys = gameObject.GetComponent<PhysicalBehaviour>();
            Car = gameObject.GetComponent<CarBehaviour>();
            for(int i = 0; i < Headlights.Length; i++)
            {
                Headlights[i].SetActive(false);
            }
            if(ReversingLamp != null)
            {
                ReversingLamp.SetActive(false);
            }
            if(firstSpawn)
            {
                startMotorSpeed = Car.MotorSpeed;
            }
            phys.ContextMenuOptions.Buttons.Add(new ContextMenuButton("cruiseControl", "Set speed limit", "Sets speed limit", new UnityAction[1]
            {
                () =>
                {
                    DialogBox dialog = null;
                    dialog = DialogBoxManager.TextEntry("Set the speed limit\n<color=green><size=26>Currently: " + Mathf.Abs(Car.MotorSpeed).ToString() + "</size></color>", "Speed limit", new DialogButton[]
                    {
                        new DialogButton("Apply", true, new UnityAction[1]
                        {
                            () =>
                            {
                                float speedLimit;
                                if(float.TryParse(dialog.EnteredText, out speedLimit))
                                {
                                    if(speedLimit > 0f)
                                    {
                                        if(speedLimit >= Mathf.Abs(startMotorSpeed))
                                        {
                                            Car.MotorSpeed = (Car.MotorSpeed > 0f) ? Mathf.Abs(startMotorSpeed) : startMotorSpeed;
                                            return;
                                        }
                                        if(Car.MotorSpeed > 0f)
                                        {
                                            Car.MotorSpeed = speedLimit;
                                        }
                                        else
                                        {
                                            Car.MotorSpeed = -speedLimit;
                                        }
                                    }
                                    else
                                    {
                                        if(speedLimit <= startMotorSpeed)
                                        {
                                            Car.MotorSpeed = (Car.MotorSpeed > 0f) ? Mathf.Abs(startMotorSpeed) : startMotorSpeed;
                                            return;
                                        }
                                        if(Car.MotorSpeed > 0f)
                                        {
                                            Car.MotorSpeed = Mathf.Abs(speedLimit);
                                        }
                                        else
                                        {
                                            Car.MotorSpeed = speedLimit;
                                        }
                                    }
                                }
                            }
                        }),
                        new DialogButton("Off", true, new UnityAction[1]
                        {
                            () =>
                            {
                                Car.MotorSpeed = startMotorSpeed;
                            }
                        }),
                        new DialogButton("Cancel", true, () => dialog.Close())
                    });
                }
            }));
            phys.ContextMenuOptions.Buttons.Add(new ContextMenuButton("toggleHeadlights", () =>
            {
                if(!HeadlightsActivated)
                {
                    return "Enable headlights";
                }
                return "Disable headlights";
            }, "Toggles headlights", new UnityAction[1]
            {
                () =>
                {
                    HeadlightsActivated = !HeadlightsActivated;
                }
            }));
            if(TurnSignals.Length != 0)
            {
                phys.ContextMenuOptions.Buttons.Add(new ContextMenuButton("toggleTurnSignals", (() =>
                {
                    if(!TurnSignalsActivated)
                    {
                        return "Enable turn signals";
                    }
                    return "Disable turn signals";
                }), "Toggles turn signals", new UnityAction[1]
                {
                    () =>
                    {
                        TurnSignalsActivated = !TurnSignalsActivated;
                        UpdateActivation();
                    }
                }));
            }
            if(FailedEngineStartup)
            {
                carAudioSources = Car.audioSourcePosition.gameObject.GetComponents<AudioSource>();
                failedEngineStartupSource = Car.audioSourcePosition.gameObject.AddComponent<AudioSource>();
                failedEngineStartupSource.spread = 0f;
                failedEngineStartupSource.volume = 1f;
                failedEngineStartupSource.minDistance = 10f;
                failedEngineStartupSource.spatialBlend = 1f;
                failedEngineStartupSource.dopplerLevel = 1f;
                gameObject.AddComponent<AudioSourceTimeScaleBehaviour>();
            }
            UpdateActivation();
            firstSpawn = false;
        }
        private void Use()
        {
            if(FailedEngineStartup)
            {
                if(Car.Activated)
                {
                    if((double)UnityEngine.Random.value > FailedEngineStartupChance)
                    {
                        carAudioSources[0].Stop();
                        carAudioSources[1].Stop();
                        Car.Activated = false;
                        failedEngineStartupSource.PlayOneShot(FailedEngineStartupSound, 1f);
                    }
                }
            }
        }
        private void Update()
        {
            for(int i = 0; i < Headlights.Length; i++)
            {
                if(HeadlightsActivated)
                {
                    Headlights[i].SetActive(true);
                }
                if(!HeadlightsActivated)
                {
                    Headlights[i].SetActive(false);
                }
            }
            if(ReversingLamp != null)
            {
                if(Car.MotorSpeed > 0 && Car.Activated && !ReversingLampActivated)
                {
                    ReversingLampActivated = true;
                    ReversingLamp.SetActive(true);
                }
                if(Car.MotorSpeed < 0 && ReversingLampActivated || !Car.Activated)
                {
                    ReversingLampActivated = false;
                    ReversingLamp.SetActive(false);
                }
            }
        }
        private void UpdateActivation()
        {
            if(TurnSignals.Length != 0)
            {
                if(TurnSignalsActivated)
                {
                    StopAllCoroutines();
                    StartCoroutine(Blink());
                }
                else
                {
                    for(int i = 0; i < TurnSignals.Length; i++)
                    {
                        TurnSignals[i].SetActive(false);
                    }
                    StopAllCoroutines();
                }
            }
        }
        private void OnCollisionEnter2D(Collision2D collision)
        {
            if(ActivateTurnSignalsOnCollision && TurnSignals.Length != 0)
            {
                if(collision.GetContact(0).normalImpulse > BreakingThreshold)
                {
                    TurnSignalsActivated = true;
                    UpdateActivation();
                }
            }
        }
        private IEnumerator Blink()
        {
            while(TurnSignalsActivated)
            {
                yield return new WaitForSeconds(0.3f);
                for(int i = 0; i < TurnSignals.Length; i++)
                {
                    TurnSignals[i].SetActive(true);
                }
                yield return new WaitForSeconds(0.3f);
                for(int i = 0; i < TurnSignals.Length; i++)
                {
                    TurnSignals[i].SetActive(false);
                }
            }
        }

        public bool firstSpawn = true;
        public bool HeadlightsActivated;
        public bool TurnSignalsActivated;
        public bool ReversingLampActivated;
        public bool ActivateTurnSignalsOnCollision;
        public bool FailedEngineStartup;
        public GameObject[] Headlights = new GameObject[0];
        public GameObject[] TurnSignals = new GameObject[0];
        public GameObject ReversingLamp;
        private PhysicalBehaviour phys;
        private AudioSource[] carAudioSources;
        private AudioSource failedEngineStartupSource;
        [SkipSerialisation]
        public CarBehaviour Car;
        [SkipSerialisation]
        public AudioClip FailedEngineStartupSound;
        public float BreakingThreshold = 20f;
        public float startMotorSpeed;
        public double FailedEngineStartupChance = 0.8d;
    }
    public class RandomCarTextureChildBehaviour : MonoBehaviour
    {
        private void Start()
        {
            phys = gameObject.GetComponent<PhysicalBehaviour>();
            renderer = gameObject.GetComponent<SpriteRenderer>();
            randomCarTextures = transform.root.gameObject.GetComponentInChildren<RandomCarTextureBehaviour>();
            if(randomCarTextures)
            {
                randomCarTextures.OnAfterChange.AddListener(new UnityAction(SyncSprite));
                randomCarTextures.OnAfterChange.Invoke();
            }
        }
        private void SyncSprite()
        {
            renderer.sprite = Sprites[randomCarTextures.chosenIndex];
            if(phys)
            {
                phys.RefreshOutline();
            }
        }

        private PhysicalBehaviour phys;
        private SpriteRenderer renderer;
        private RandomCarTextureBehaviour randomCarTextures;
        public Sprite[] Sprites = new Sprite[0];
    }
    public class TowbarBehaviour : MonoBehaviour
    {
        public interface IOnConnected
        {
            void OnConnected(TowbarBehaviour otherTowbar);
        }
        public interface IOnDisconnected
        {
            void OnDisconnected();
        }
        private void Start()
        {
            gameObject.GetComponent<PhysicalBehaviour>().ContextMenuOptions.Buttons.Add(new ContextMenuButton("disconnect", "Disconnect", "Disconnect", new UnityAction[]
            {
                delegate()
                {
                    if(fixedJoint)
                    {
                        Disconnect();
                    }
                    else
                    {
                        connectedTowbar.Disconnect();
                    }
                }
            })
            {
                Condition = (() => connectedTowbar)
            });
        }
        private void OnCollisionEnter2D(Collision2D collision)
        {
            if(collision.gameObject.GetComponent<TowbarBehaviour>() && !connectedTowbar && !collision.gameObject.GetComponent<TowbarBehaviour>().connectedTowbar && collision.gameObject.GetComponent<TowbarBehaviour>().IsTrailerTowbar != IsTrailerTowbar)
            {
                connectedTowbar = collision.gameObject.GetComponent<TowbarBehaviour>();
                collision.gameObject.GetComponent<TowbarBehaviour>().connectedTowbar = gameObject.GetComponent<TowbarBehaviour>();
                fixedJoint = gameObject.AddComponent<FixedJoint2D>();
                fixedJoint.breakForce = 40000f;
                fixedJoint.connectedBody = collision.gameObject.GetComponent<Rigidbody2D>();
                fixedJoint.anchor = Anchor;
                fixedJoint.frequency = 10f;
                fixedJoint.dampingRatio = 1f;
                noCollide = gameObject.AddComponent<NoCollide>();
                noCollide.NoCollideSetA = GetComponents<Collider2D>();
                noCollide.NoCollideSetB = collision.gameObject.GetComponents<Collider2D>();
                transform.root.gameObject.BroadcastMessage("OnConnected", connectedTowbar, SendMessageOptions.DontRequireReceiver);
                collision.transform.root.gameObject.BroadcastMessage("OnConnected", this, SendMessageOptions.DontRequireReceiver);
            }
        }
        private void OnJointBreak2D(Joint2D joint)
        {
            if (noCollide)
            {
                Disconnect();
            }
            else
            {
                connectedTowbar.Disconnect();
            }
        }
        public void Disconnect()
        {
            transform.root.gameObject.BroadcastMessage("OnDisconnected", connectedTowbar, SendMessageOptions.DontRequireReceiver);
            connectedTowbar.transform.root.gameObject.BroadcastMessage("OnDisconnected", this, SendMessageOptions.DontRequireReceiver);
            Destroy(fixedJoint);
            Destroy(noCollide, 2.5f);
            connectedTowbar.connectedTowbar = null;
            connectedTowbar = null;
        }

        public bool IsTrailerTowbar = false;
        [SkipSerialisation]
        public TowbarBehaviour connectedTowbar;
        [SkipSerialisation]
        public FixedJoint2D fixedJoint;
        [SkipSerialisation]
        public NoCollide noCollide;
        [SkipSerialisation]
        public Vector2 Anchor = Vector2.zero;
    }
    public class TrailerBehaviour : MonoBehaviour, TowbarBehaviour.IOnConnected, TowbarBehaviour.IOnDisconnected
    {
        private void Start()
        {
            phys = gameObject.GetComponent<PhysicalBehaviour>();
            phys.ContextMenuOptions.Buttons.Add(new ContextMenuButton("disconnect", "Disconnect", "Disconnect", new UnityAction[1]
            {
                () =>
                {
                    if (TowbarBehaviour.fixedJoint)
                    {
                        TowbarBehaviour.Disconnect();
                    }
                    else
                    {
                        TowbarBehaviour.connectedTowbar.Disconnect();
                    }
                }
            }));
            StartCoroutine(SupportFoot.Move(1f));
        }
        public void OnConnected(TowbarBehaviour towbarBehaviour)
        {
            carBehaviour = towbarBehaviour.transform.root.gameObject.GetComponentInChildren<CarBehaviour>();
            List<WheelJoint2D> wheelList = carBehaviour.WheelJoints.ToList();
            wheelList.AddRange(Wheels);
            carBehaviour.WheelJoints = wheelList.ToArray();
            StartCoroutine(SupportFoot.Move(-1f));
        }
        public void OnDisconnected()
        {
            List<WheelJoint2D> wheelList = carBehaviour.WheelJoints.ToList();
            foreach (WheelJoint2D wheel in Wheels)
            {
                float maxMotorTorque = wheel.motor.maxMotorTorque;
                wheel.motor = new JointMotor2D
                {
                    motorSpeed = 0f,
                    maxMotorTorque = maxMotorTorque
                };
                wheelList.Remove(wheel);
            }
            carBehaviour.WheelJoints = wheelList.ToArray();
            carBehaviour = null;
            StartCoroutine(SupportFoot.Move(1f));
        }
        private void OnJointBreak2D(Joint2D joint)
        {
            if (!Wheels.Contains(joint))
            {
                return;
            }
            List<WheelJoint2D> wheels = Wheels.ToList();
            wheels.Remove(joint as WheelJoint2D);
            Wheels = wheels.ToArray();
            if(carBehaviour != null)
            {
                List<WheelJoint2D> wheelList = carBehaviour.WheelJoints.ToList();
                wheelList.Remove(joint as WheelJoint2D);
                carBehaviour.WheelJoints = wheelList.ToArray();
            }
        }

        private bool canExplode;
        private PhysicalBehaviour phys;
        [SkipSerialisation]
        private CarBehaviour carBehaviour;
        [SkipSerialisation]
        public TowbarBehaviour TowbarBehaviour;
        [SkipSerialisation]
        public WheelJoint2D[] Wheels;
        [SkipSerialisation]
        public SupportFootBehaviour SupportFoot;
    }
    public class SupportFootBehaviour : MonoBehaviour
    {
        private void Start()
        {
            originalLimits = SliderJoint.limits;
            UpdateLimits();
        }
        public IEnumerator Move(float direction)
        {
            JointLimitState2D limitToWaitFor = (direction > 0f) ? JointLimitState2D.UpperLimit : JointLimitState2D.LowerLimit;
            SetMotorSpeed(direction * Mathf.Abs(gameObject.transform.localScale.x));
            yield return new WaitUntil(() => SliderJoint.limitState == limitToWaitFor);
            SetMotorSpeed(0f);
            yield break;
        }
        private void Update()
        {
            UpdateLimits();
        }
        private void UpdateLimits()
        {
            JointTranslationLimits2D limits = default;
            float num = Mathf.Abs(gameObject.transform.localScale.x);
            limits.max = originalLimits.max * num;
            limits.min = originalLimits.min * num;
            SliderJoint.limits = limits;
        }
        private void SetMotorSpeed(float speed)
        {
            JointMotor2D motor = SliderJoint.motor;
            motor.motorSpeed = speed * 0.5f;
            SliderJoint.motor = motor;
        }

        [SkipSerialisation]
        public SliderJoint2D SliderJoint;
        private JointTranslationLimits2D originalLimits;
    }
    public class BlinkerBehaviour : MonoBehaviour
    {
        private void Start()
        {
            Glow1.SetActive(false);
            Glow2.SetActive(false);
            damagableMachinery = gameObject.GetComponent<DamagableMachineryBehaviour>();
            damagableMachinery.OnDamaged = new UnityEvent();
            damagableMachinery.OnDamaged.AddListener(() =>
            {
                UpdateActivation();
            });
            UpdateActivation();
        }
        private void Use()
        {
            Activated = !Activated;
            UpdateActivation();
        }
        private void UpdateActivation()
        {
            if(Activated && !damagableMachinery.Destroyed)
            {
                Audio.Play();
                StartCoroutine(Blink(Delay));
            }
            else
            {
                Audio.Stop();
                StopAllCoroutines();
                Glow1.SetActive(false);
                Glow2.SetActive(false);
            }
            if(damagableMachinery.Destroyed)
            {
                Audio.Stop();
                StopAllCoroutines();
                Glow1.SetActive(false);
                Glow2.SetActive(false);
                Activated = false;
            }
        }
        private IEnumerator Blink(float delay)
        {
            Glow1.SetActive(true);
            while(Activated)
            {
                yield return new WaitForSeconds(delay);
                Glow1.SetActive(false);
                Glow2.SetActive(true);
                yield return new WaitForSeconds(delay);
                Glow1.SetActive(true);
                Glow2.SetActive(false);
            }
        }

        public bool Activated;
        public GameObject Glow1;
        public GameObject Glow2;
        public AudioSource Audio;
        private DamagableMachineryBehaviour damagableMachinery;
        public float Delay;
    }
    public class Gasoline : Liquid
    {
        public Gasoline()
        {
            Color = new Color32(237, 255, 33, 255);
        }
        public override void OnEnterContainer(BloodContainer container)
        {
            CarBehaviour carBehaviour = container.transform.root.GetComponentInChildren<CarBehaviour>();
            if (carBehaviour)
            {
                carBehaviour.MotorSpeed = (carBehaviour.MotorSpeed > 0) ? carBehaviour.MotorSpeed + 750f : carBehaviour.MotorSpeed - 750f;
            }
        }
        public override void OnEnterLimb(LimbBehaviour limb)
        {
            limb.PhysicalBehaviour.Ignite();
        }
        public override void OnExitContainer(BloodContainer container)
        {
            CarBehaviour carBehaviour = container.transform.root.gameObject.GetComponentInChildren<CarBehaviour>();
            if (carBehaviour)
            {
                carBehaviour.MotorSpeed = (carBehaviour.MotorSpeed > 0) ? carBehaviour.MotorSpeed - 750f : carBehaviour.MotorSpeed + 750f;
            }
        }
    }
}